from enum import Enum
import math
import string


class ReactiveEnergyUnits(Enum):
        """
            ReactiveEnergyUnits enumeration
        """
        
        VoltampereReactiveHour = 'voltampere_reactive_hour'
        """
            
        """
        
        KilovoltampereReactiveHour = 'kilovoltampere_reactive_hour'
        """
            
        """
        
        MegavoltampereReactiveHour = 'megavoltampere_reactive_hour'
        """
            
        """
        

class ReactiveEnergy:
    """
    The Volt-ampere reactive hour (expressed as varh) is the reactive power of one Volt-ampere reactive produced in one hour.

    Args:
        value (float): The value.
        from_unit (ReactiveEnergyUnits): The ReactiveEnergy unit to create from, The default unit is VoltampereReactiveHour
    """
    def __init__(self, value: float, from_unit: ReactiveEnergyUnits = ReactiveEnergyUnits.VoltampereReactiveHour):
        if math.isnan(value):
            raise ValueError('Invalid unit: value is NaN')
        self.__value = self.__convert_to_base(value, from_unit)
        
        self.__voltampere_reactive_hours = None
        
        self.__kilovoltampere_reactive_hours = None
        
        self.__megavoltampere_reactive_hours = None
        

    def __convert_from_base(self, from_unit: ReactiveEnergyUnits) -> float:
        value = self.__value
        
        if from_unit == ReactiveEnergyUnits.VoltampereReactiveHour:
            return (value)
        
        if from_unit == ReactiveEnergyUnits.KilovoltampereReactiveHour:
            return ((value) / 1000.0)
        
        if from_unit == ReactiveEnergyUnits.MegavoltampereReactiveHour:
            return ((value) / 1000000.0)
        
        return None


    def __convert_to_base(self, value: float, to_unit: ReactiveEnergyUnits) -> float:
        
        if to_unit == ReactiveEnergyUnits.VoltampereReactiveHour:
            return (value)
        
        if to_unit == ReactiveEnergyUnits.KilovoltampereReactiveHour:
            return ((value) * 1000.0)
        
        if to_unit == ReactiveEnergyUnits.MegavoltampereReactiveHour:
            return ((value) * 1000000.0)
        
        return None


    @property
    def base_value(self) -> float:
        return self.__value

    
    @staticmethod
    def from_voltampere_reactive_hours(voltampere_reactive_hours: float):
        """
        Create a new instance of ReactiveEnergy from a value in voltampere_reactive_hours.

        

        :param meters: The ReactiveEnergy value in voltampere_reactive_hours.
        :type voltampere_reactive_hours: float
        :return: A new instance of ReactiveEnergy.
        :rtype: ReactiveEnergy
        """
        return ReactiveEnergy(voltampere_reactive_hours, ReactiveEnergyUnits.VoltampereReactiveHour)

    
    @staticmethod
    def from_kilovoltampere_reactive_hours(kilovoltampere_reactive_hours: float):
        """
        Create a new instance of ReactiveEnergy from a value in kilovoltampere_reactive_hours.

        

        :param meters: The ReactiveEnergy value in kilovoltampere_reactive_hours.
        :type kilovoltampere_reactive_hours: float
        :return: A new instance of ReactiveEnergy.
        :rtype: ReactiveEnergy
        """
        return ReactiveEnergy(kilovoltampere_reactive_hours, ReactiveEnergyUnits.KilovoltampereReactiveHour)

    
    @staticmethod
    def from_megavoltampere_reactive_hours(megavoltampere_reactive_hours: float):
        """
        Create a new instance of ReactiveEnergy from a value in megavoltampere_reactive_hours.

        

        :param meters: The ReactiveEnergy value in megavoltampere_reactive_hours.
        :type megavoltampere_reactive_hours: float
        :return: A new instance of ReactiveEnergy.
        :rtype: ReactiveEnergy
        """
        return ReactiveEnergy(megavoltampere_reactive_hours, ReactiveEnergyUnits.MegavoltampereReactiveHour)

    
    @property
    def voltampere_reactive_hours(self) -> float:
        """
        
        """
        if self.__voltampere_reactive_hours != None:
            return self.__voltampere_reactive_hours
        self.__voltampere_reactive_hours = self.__convert_from_base(ReactiveEnergyUnits.VoltampereReactiveHour)
        return self.__voltampere_reactive_hours

    
    @property
    def kilovoltampere_reactive_hours(self) -> float:
        """
        
        """
        if self.__kilovoltampere_reactive_hours != None:
            return self.__kilovoltampere_reactive_hours
        self.__kilovoltampere_reactive_hours = self.__convert_from_base(ReactiveEnergyUnits.KilovoltampereReactiveHour)
        return self.__kilovoltampere_reactive_hours

    
    @property
    def megavoltampere_reactive_hours(self) -> float:
        """
        
        """
        if self.__megavoltampere_reactive_hours != None:
            return self.__megavoltampere_reactive_hours
        self.__megavoltampere_reactive_hours = self.__convert_from_base(ReactiveEnergyUnits.MegavoltampereReactiveHour)
        return self.__megavoltampere_reactive_hours

    
    def to_string(self, unit: ReactiveEnergyUnits = ReactiveEnergyUnits.VoltampereReactiveHour) -> string:
        """
        Format the ReactiveEnergy to string.
        Note! the default format for ReactiveEnergy is VoltampereReactiveHour.
        To specify the unit format set the 'unit' parameter.
        """
        
        if unit == ReactiveEnergyUnits.VoltampereReactiveHour:
            return f"""{self.voltampere_reactive_hours} varh"""
        
        if unit == ReactiveEnergyUnits.KilovoltampereReactiveHour:
            return f"""{self.kilovoltampere_reactive_hours} """
        
        if unit == ReactiveEnergyUnits.MegavoltampereReactiveHour:
            return f"""{self.megavoltampere_reactive_hours} """
        
        return f'{self.__value}'


    def get_unit_abbreviation(self, unit_abbreviation: ReactiveEnergyUnits = ReactiveEnergyUnits.VoltampereReactiveHour) -> string:
        """
        Get ReactiveEnergy unit abbreviation.
        Note! the default abbreviation for ReactiveEnergy is VoltampereReactiveHour.
        To specify the unit abbreviation set the 'unit_abbreviation' parameter.
        """
        
        if unit_abbreviation == ReactiveEnergyUnits.VoltampereReactiveHour:
            return """varh"""
        
        if unit_abbreviation == ReactiveEnergyUnits.KilovoltampereReactiveHour:
            return """"""
        
        if unit_abbreviation == ReactiveEnergyUnits.MegavoltampereReactiveHour:
            return """"""
        

    def __str__(self):
        return self.to_string()


    def __add__(self, other):
        if not isinstance(other, ReactiveEnergy):
            raise TypeError("unsupported operand type(s) for +: 'ReactiveEnergy' and '{}'".format(type(other).__name__))
        return ReactiveEnergy(self.__value + other.__value)


    def __mul__(self, other):
        if not isinstance(other, ReactiveEnergy):
            raise TypeError("unsupported operand type(s) for *: 'ReactiveEnergy' and '{}'".format(type(other).__name__))
        return ReactiveEnergy(self.__value * other.__value)


    def __sub__(self, other):
        if not isinstance(other, ReactiveEnergy):
            raise TypeError("unsupported operand type(s) for -: 'ReactiveEnergy' and '{}'".format(type(other).__name__))
        return ReactiveEnergy(self.__value - other.__value)


    def __truediv__(self, other):
        if not isinstance(other, ReactiveEnergy):
            raise TypeError("unsupported operand type(s) for /: 'ReactiveEnergy' and '{}'".format(type(other).__name__))
        return ReactiveEnergy(self.__value / other.__value)


    def __mod__(self, other):
        if not isinstance(other, ReactiveEnergy):
            raise TypeError("unsupported operand type(s) for %: 'ReactiveEnergy' and '{}'".format(type(other).__name__))
        return ReactiveEnergy(self.__value % other.__value)


    def __pow__(self, other):
        if not isinstance(other, ReactiveEnergy):
            raise TypeError("unsupported operand type(s) for **: 'ReactiveEnergy' and '{}'".format(type(other).__name__))
        return ReactiveEnergy(self.__value ** other.__value)


    def __eq__(self, other):
        if not isinstance(other, ReactiveEnergy):
            raise TypeError("unsupported operand type(s) for ==: 'ReactiveEnergy' and '{}'".format(type(other).__name__))
        return self.__value == other.__value


    def __lt__(self, other):
        if not isinstance(other, ReactiveEnergy):
            raise TypeError("unsupported operand type(s) for <: 'ReactiveEnergy' and '{}'".format(type(other).__name__))
        return self.__value < other.__value


    def __gt__(self, other):
        if not isinstance(other, ReactiveEnergy):
            raise TypeError("unsupported operand type(s) for >: 'ReactiveEnergy' and '{}'".format(type(other).__name__))
        return self.__value > other.__value


    def __le__(self, other):
        if not isinstance(other, ReactiveEnergy):
            raise TypeError("unsupported operand type(s) for <=: 'ReactiveEnergy' and '{}'".format(type(other).__name__))
        return self.__value <= other.__value


    def __ge__(self, other):
        if not isinstance(other, ReactiveEnergy):
            raise TypeError("unsupported operand type(s) for >=: 'ReactiveEnergy' and '{}'".format(type(other).__name__))
        return self.__value >= other.__value


from enum import Enum
import math
import string


class ScalarUnits(Enum):
        """
            ScalarUnits enumeration
        """
        
        Amount = 'amount'
        """
            
        """
        

class Scalar:
    """
    A way of representing a number of items.

    Args:
        value (float): The value.
        from_unit (ScalarUnits): The Scalar unit to create from, The default unit is Amount
    """
    def __init__(self, value: float, from_unit: ScalarUnits = ScalarUnits.Amount):
        if math.isnan(value):
            raise ValueError('Invalid unit: value is NaN')
        self.__value = self.__convert_to_base(value, from_unit)
        
        self.__amount = None
        

    def __convert_from_base(self, from_unit: ScalarUnits) -> float:
        value = self.__value
        
        if from_unit == ScalarUnits.Amount:
            return (value)
        
        return None


    def __convert_to_base(self, value: float, to_unit: ScalarUnits) -> float:
        
        if to_unit == ScalarUnits.Amount:
            return (value)
        
        return None


    @property
    def base_value(self) -> float:
        return self.__value

    
    @staticmethod
    def from_amount(amount: float):
        """
        Create a new instance of Scalar from a value in amount.

        

        :param meters: The Scalar value in amount.
        :type amount: float
        :return: A new instance of Scalar.
        :rtype: Scalar
        """
        return Scalar(amount, ScalarUnits.Amount)

    
    @property
    def amount(self) -> float:
        """
        
        """
        if self.__amount != None:
            return self.__amount
        self.__amount = self.__convert_from_base(ScalarUnits.Amount)
        return self.__amount

    
    def to_string(self, unit: ScalarUnits = ScalarUnits.Amount) -> string:
        """
        Format the Scalar to string.
        Note! the default format for Scalar is Amount.
        To specify the unit format set the 'unit' parameter.
        """
        
        if unit == ScalarUnits.Amount:
            return f"""{self.amount} """
        
        return f'{self.__value}'


    def get_unit_abbreviation(self, unit_abbreviation: ScalarUnits = ScalarUnits.Amount) -> string:
        """
        Get Scalar unit abbreviation.
        Note! the default abbreviation for Scalar is Amount.
        To specify the unit abbreviation set the 'unit_abbreviation' parameter.
        """
        
        if unit_abbreviation == ScalarUnits.Amount:
            return """"""
        

    def __str__(self):
        return self.to_string()


    def __add__(self, other):
        if not isinstance(other, Scalar):
            raise TypeError("unsupported operand type(s) for +: 'Scalar' and '{}'".format(type(other).__name__))
        return Scalar(self.__value + other.__value)


    def __mul__(self, other):
        if not isinstance(other, Scalar):
            raise TypeError("unsupported operand type(s) for *: 'Scalar' and '{}'".format(type(other).__name__))
        return Scalar(self.__value * other.__value)


    def __sub__(self, other):
        if not isinstance(other, Scalar):
            raise TypeError("unsupported operand type(s) for -: 'Scalar' and '{}'".format(type(other).__name__))
        return Scalar(self.__value - other.__value)


    def __truediv__(self, other):
        if not isinstance(other, Scalar):
            raise TypeError("unsupported operand type(s) for /: 'Scalar' and '{}'".format(type(other).__name__))
        return Scalar(self.__value / other.__value)


    def __mod__(self, other):
        if not isinstance(other, Scalar):
            raise TypeError("unsupported operand type(s) for %: 'Scalar' and '{}'".format(type(other).__name__))
        return Scalar(self.__value % other.__value)


    def __pow__(self, other):
        if not isinstance(other, Scalar):
            raise TypeError("unsupported operand type(s) for **: 'Scalar' and '{}'".format(type(other).__name__))
        return Scalar(self.__value ** other.__value)


    def __eq__(self, other):
        if not isinstance(other, Scalar):
            raise TypeError("unsupported operand type(s) for ==: 'Scalar' and '{}'".format(type(other).__name__))
        return self.__value == other.__value


    def __lt__(self, other):
        if not isinstance(other, Scalar):
            raise TypeError("unsupported operand type(s) for <: 'Scalar' and '{}'".format(type(other).__name__))
        return self.__value < other.__value


    def __gt__(self, other):
        if not isinstance(other, Scalar):
            raise TypeError("unsupported operand type(s) for >: 'Scalar' and '{}'".format(type(other).__name__))
        return self.__value > other.__value


    def __le__(self, other):
        if not isinstance(other, Scalar):
            raise TypeError("unsupported operand type(s) for <=: 'Scalar' and '{}'".format(type(other).__name__))
        return self.__value <= other.__value


    def __ge__(self, other):
        if not isinstance(other, Scalar):
            raise TypeError("unsupported operand type(s) for >=: 'Scalar' and '{}'".format(type(other).__name__))
        return self.__value >= other.__value


from enum import Enum
import math
import string


class PermeabilityUnits(Enum):
        """
            PermeabilityUnits enumeration
        """
        
        HenryPerMeter = 'henry_per_meter'
        """
            
        """
        

class Permeability:
    """
    In electromagnetism, permeability is the measure of the ability of a material to support the formation of a magnetic field within itself.

    Args:
        value (float): The value.
        from_unit (PermeabilityUnits): The Permeability unit to create from, The default unit is HenryPerMeter
    """
    def __init__(self, value: float, from_unit: PermeabilityUnits = PermeabilityUnits.HenryPerMeter):
        if math.isnan(value):
            raise ValueError('Invalid unit: value is NaN')
        self.__value = self.__convert_to_base(value, from_unit)
        
        self.__henries_per_meter = None
        

    def __convert_from_base(self, from_unit: PermeabilityUnits) -> float:
        value = self.__value
        
        if from_unit == PermeabilityUnits.HenryPerMeter:
            return (value)
        
        return None


    def __convert_to_base(self, value: float, to_unit: PermeabilityUnits) -> float:
        
        if to_unit == PermeabilityUnits.HenryPerMeter:
            return (value)
        
        return None


    @property
    def base_value(self) -> float:
        return self.__value

    
    @staticmethod
    def from_henries_per_meter(henries_per_meter: float):
        """
        Create a new instance of Permeability from a value in henries_per_meter.

        

        :param meters: The Permeability value in henries_per_meter.
        :type henries_per_meter: float
        :return: A new instance of Permeability.
        :rtype: Permeability
        """
        return Permeability(henries_per_meter, PermeabilityUnits.HenryPerMeter)

    
    @property
    def henries_per_meter(self) -> float:
        """
        
        """
        if self.__henries_per_meter != None:
            return self.__henries_per_meter
        self.__henries_per_meter = self.__convert_from_base(PermeabilityUnits.HenryPerMeter)
        return self.__henries_per_meter

    
    def to_string(self, unit: PermeabilityUnits = PermeabilityUnits.HenryPerMeter) -> string:
        """
        Format the Permeability to string.
        Note! the default format for Permeability is HenryPerMeter.
        To specify the unit format set the 'unit' parameter.
        """
        
        if unit == PermeabilityUnits.HenryPerMeter:
            return f"""{self.henries_per_meter} H/m"""
        
        return f'{self.__value}'


    def get_unit_abbreviation(self, unit_abbreviation: PermeabilityUnits = PermeabilityUnits.HenryPerMeter) -> string:
        """
        Get Permeability unit abbreviation.
        Note! the default abbreviation for Permeability is HenryPerMeter.
        To specify the unit abbreviation set the 'unit_abbreviation' parameter.
        """
        
        if unit_abbreviation == PermeabilityUnits.HenryPerMeter:
            return """H/m"""
        

    def __str__(self):
        return self.to_string()


    def __add__(self, other):
        if not isinstance(other, Permeability):
            raise TypeError("unsupported operand type(s) for +: 'Permeability' and '{}'".format(type(other).__name__))
        return Permeability(self.__value + other.__value)


    def __mul__(self, other):
        if not isinstance(other, Permeability):
            raise TypeError("unsupported operand type(s) for *: 'Permeability' and '{}'".format(type(other).__name__))
        return Permeability(self.__value * other.__value)


    def __sub__(self, other):
        if not isinstance(other, Permeability):
            raise TypeError("unsupported operand type(s) for -: 'Permeability' and '{}'".format(type(other).__name__))
        return Permeability(self.__value - other.__value)


    def __truediv__(self, other):
        if not isinstance(other, Permeability):
            raise TypeError("unsupported operand type(s) for /: 'Permeability' and '{}'".format(type(other).__name__))
        return Permeability(self.__value / other.__value)


    def __mod__(self, other):
        if not isinstance(other, Permeability):
            raise TypeError("unsupported operand type(s) for %: 'Permeability' and '{}'".format(type(other).__name__))
        return Permeability(self.__value % other.__value)


    def __pow__(self, other):
        if not isinstance(other, Permeability):
            raise TypeError("unsupported operand type(s) for **: 'Permeability' and '{}'".format(type(other).__name__))
        return Permeability(self.__value ** other.__value)


    def __eq__(self, other):
        if not isinstance(other, Permeability):
            raise TypeError("unsupported operand type(s) for ==: 'Permeability' and '{}'".format(type(other).__name__))
        return self.__value == other.__value


    def __lt__(self, other):
        if not isinstance(other, Permeability):
            raise TypeError("unsupported operand type(s) for <: 'Permeability' and '{}'".format(type(other).__name__))
        return self.__value < other.__value


    def __gt__(self, other):
        if not isinstance(other, Permeability):
            raise TypeError("unsupported operand type(s) for >: 'Permeability' and '{}'".format(type(other).__name__))
        return self.__value > other.__value


    def __le__(self, other):
        if not isinstance(other, Permeability):
            raise TypeError("unsupported operand type(s) for <=: 'Permeability' and '{}'".format(type(other).__name__))
        return self.__value <= other.__value


    def __ge__(self, other):
        if not isinstance(other, Permeability):
            raise TypeError("unsupported operand type(s) for >=: 'Permeability' and '{}'".format(type(other).__name__))
        return self.__value >= other.__value


from enum import Enum
import math
import string


class PowerUnits(Enum):
        """
            PowerUnits enumeration
        """
        
        Watt = 'watt'
        """
            
        """
        
        MechanicalHorsepower = 'mechanical_horsepower'
        """
            
        """
        
        MetricHorsepower = 'metric_horsepower'
        """
            
        """
        
        ElectricalHorsepower = 'electrical_horsepower'
        """
            
        """
        
        BoilerHorsepower = 'boiler_horsepower'
        """
            
        """
        
        HydraulicHorsepower = 'hydraulic_horsepower'
        """
            
        """
        
        BritishThermalUnitPerHour = 'british_thermal_unit_per_hour'
        """
            
        """
        
        JoulePerHour = 'joule_per_hour'
        """
            
        """
        
        Femtowatt = 'femtowatt'
        """
            
        """
        
        Picowatt = 'picowatt'
        """
            
        """
        
        Nanowatt = 'nanowatt'
        """
            
        """
        
        Microwatt = 'microwatt'
        """
            
        """
        
        Milliwatt = 'milliwatt'
        """
            
        """
        
        Deciwatt = 'deciwatt'
        """
            
        """
        
        Decawatt = 'decawatt'
        """
            
        """
        
        Kilowatt = 'kilowatt'
        """
            
        """
        
        Megawatt = 'megawatt'
        """
            
        """
        
        Gigawatt = 'gigawatt'
        """
            
        """
        
        Terawatt = 'terawatt'
        """
            
        """
        
        Petawatt = 'petawatt'
        """
            
        """
        
        KilobritishThermalUnitPerHour = 'kilobritish_thermal_unit_per_hour'
        """
            
        """
        
        MegabritishThermalUnitPerHour = 'megabritish_thermal_unit_per_hour'
        """
            
        """
        
        MillijoulePerHour = 'millijoule_per_hour'
        """
            
        """
        
        KilojoulePerHour = 'kilojoule_per_hour'
        """
            
        """
        
        MegajoulePerHour = 'megajoule_per_hour'
        """
            
        """
        
        GigajoulePerHour = 'gigajoule_per_hour'
        """
            
        """
        

class Power:
    """
    In physics, power is the rate of doing work. It is equivalent to an amount of energy consumed per unit time.

    Args:
        value (float): The value.
        from_unit (PowerUnits): The Power unit to create from, The default unit is Watt
    """
    def __init__(self, value: float, from_unit: PowerUnits = PowerUnits.Watt):
        if math.isnan(value):
            raise ValueError('Invalid unit: value is NaN')
        self.__value = self.__convert_to_base(value, from_unit)
        
        self.__watts = None
        
        self.__mechanical_horsepower = None
        
        self.__metric_horsepower = None
        
        self.__electrical_horsepower = None
        
        self.__boiler_horsepower = None
        
        self.__hydraulic_horsepower = None
        
        self.__british_thermal_units_per_hour = None
        
        self.__joules_per_hour = None
        
        self.__femtowatts = None
        
        self.__picowatts = None
        
        self.__nanowatts = None
        
        self.__microwatts = None
        
        self.__milliwatts = None
        
        self.__deciwatts = None
        
        self.__decawatts = None
        
        self.__kilowatts = None
        
        self.__megawatts = None
        
        self.__gigawatts = None
        
        self.__terawatts = None
        
        self.__petawatts = None
        
        self.__kilobritish_thermal_units_per_hour = None
        
        self.__megabritish_thermal_units_per_hour = None
        
        self.__millijoules_per_hour = None
        
        self.__kilojoules_per_hour = None
        
        self.__megajoules_per_hour = None
        
        self.__gigajoules_per_hour = None
        

    def __convert_from_base(self, from_unit: PowerUnits) -> float:
        value = self.__value
        
        if from_unit == PowerUnits.Watt:
            return (value)
        
        if from_unit == PowerUnits.MechanicalHorsepower:
            return (value / 745.69)
        
        if from_unit == PowerUnits.MetricHorsepower:
            return (value / 735.49875)
        
        if from_unit == PowerUnits.ElectricalHorsepower:
            return (value / 746)
        
        if from_unit == PowerUnits.BoilerHorsepower:
            return (value / 9812.5)
        
        if from_unit == PowerUnits.HydraulicHorsepower:
            return (value / 745.69988145)
        
        if from_unit == PowerUnits.BritishThermalUnitPerHour:
            return (value / 0.29307107017)
        
        if from_unit == PowerUnits.JoulePerHour:
            return (value * 3600)
        
        if from_unit == PowerUnits.Femtowatt:
            return ((value) / 1e-15)
        
        if from_unit == PowerUnits.Picowatt:
            return ((value) / 1e-12)
        
        if from_unit == PowerUnits.Nanowatt:
            return ((value) / 1e-09)
        
        if from_unit == PowerUnits.Microwatt:
            return ((value) / 1e-06)
        
        if from_unit == PowerUnits.Milliwatt:
            return ((value) / 0.001)
        
        if from_unit == PowerUnits.Deciwatt:
            return ((value) / 0.1)
        
        if from_unit == PowerUnits.Decawatt:
            return ((value) / 10.0)
        
        if from_unit == PowerUnits.Kilowatt:
            return ((value) / 1000.0)
        
        if from_unit == PowerUnits.Megawatt:
            return ((value) / 1000000.0)
        
        if from_unit == PowerUnits.Gigawatt:
            return ((value) / 1000000000.0)
        
        if from_unit == PowerUnits.Terawatt:
            return ((value) / 1000000000000.0)
        
        if from_unit == PowerUnits.Petawatt:
            return ((value) / 1000000000000000.0)
        
        if from_unit == PowerUnits.KilobritishThermalUnitPerHour:
            return ((value / 0.29307107017) / 1000.0)
        
        if from_unit == PowerUnits.MegabritishThermalUnitPerHour:
            return ((value / 0.29307107017) / 1000000.0)
        
        if from_unit == PowerUnits.MillijoulePerHour:
            return ((value * 3600) / 0.001)
        
        if from_unit == PowerUnits.KilojoulePerHour:
            return ((value * 3600) / 1000.0)
        
        if from_unit == PowerUnits.MegajoulePerHour:
            return ((value * 3600) / 1000000.0)
        
        if from_unit == PowerUnits.GigajoulePerHour:
            return ((value * 3600) / 1000000000.0)
        
        return None


    def __convert_to_base(self, value: float, to_unit: PowerUnits) -> float:
        
        if to_unit == PowerUnits.Watt:
            return (value)
        
        if to_unit == PowerUnits.MechanicalHorsepower:
            return (value * 745.69)
        
        if to_unit == PowerUnits.MetricHorsepower:
            return (value * 735.49875)
        
        if to_unit == PowerUnits.ElectricalHorsepower:
            return (value * 746)
        
        if to_unit == PowerUnits.BoilerHorsepower:
            return (value * 9812.5)
        
        if to_unit == PowerUnits.HydraulicHorsepower:
            return (value * 745.69988145)
        
        if to_unit == PowerUnits.BritishThermalUnitPerHour:
            return (value * 0.29307107017)
        
        if to_unit == PowerUnits.JoulePerHour:
            return (value / 3600)
        
        if to_unit == PowerUnits.Femtowatt:
            return ((value) * 1e-15)
        
        if to_unit == PowerUnits.Picowatt:
            return ((value) * 1e-12)
        
        if to_unit == PowerUnits.Nanowatt:
            return ((value) * 1e-09)
        
        if to_unit == PowerUnits.Microwatt:
            return ((value) * 1e-06)
        
        if to_unit == PowerUnits.Milliwatt:
            return ((value) * 0.001)
        
        if to_unit == PowerUnits.Deciwatt:
            return ((value) * 0.1)
        
        if to_unit == PowerUnits.Decawatt:
            return ((value) * 10.0)
        
        if to_unit == PowerUnits.Kilowatt:
            return ((value) * 1000.0)
        
        if to_unit == PowerUnits.Megawatt:
            return ((value) * 1000000.0)
        
        if to_unit == PowerUnits.Gigawatt:
            return ((value) * 1000000000.0)
        
        if to_unit == PowerUnits.Terawatt:
            return ((value) * 1000000000000.0)
        
        if to_unit == PowerUnits.Petawatt:
            return ((value) * 1000000000000000.0)
        
        if to_unit == PowerUnits.KilobritishThermalUnitPerHour:
            return ((value * 0.29307107017) * 1000.0)
        
        if to_unit == PowerUnits.MegabritishThermalUnitPerHour:
            return ((value * 0.29307107017) * 1000000.0)
        
        if to_unit == PowerUnits.MillijoulePerHour:
            return ((value / 3600) * 0.001)
        
        if to_unit == PowerUnits.KilojoulePerHour:
            return ((value / 3600) * 1000.0)
        
        if to_unit == PowerUnits.MegajoulePerHour:
            return ((value / 3600) * 1000000.0)
        
        if to_unit == PowerUnits.GigajoulePerHour:
            return ((value / 3600) * 1000000000.0)
        
        return None


    @property
    def base_value(self) -> float:
        return self.__value

    
    @staticmethod
    def from_watts(watts: float):
        """
        Create a new instance of Power from a value in watts.

        

        :param meters: The Power value in watts.
        :type watts: float
        :return: A new instance of Power.
        :rtype: Power
        """
        return Power(watts, PowerUnits.Watt)

    
    @staticmethod
    def from_mechanical_horsepower(mechanical_horsepower: float):
        """
        Create a new instance of Power from a value in mechanical_horsepower.

        

        :param meters: The Power value in mechanical_horsepower.
        :type mechanical_horsepower: float
        :return: A new instance of Power.
        :rtype: Power
        """
        return Power(mechanical_horsepower, PowerUnits.MechanicalHorsepower)

    
    @staticmethod
    def from_metric_horsepower(metric_horsepower: float):
        """
        Create a new instance of Power from a value in metric_horsepower.

        

        :param meters: The Power value in metric_horsepower.
        :type metric_horsepower: float
        :return: A new instance of Power.
        :rtype: Power
        """
        return Power(metric_horsepower, PowerUnits.MetricHorsepower)

    
    @staticmethod
    def from_electrical_horsepower(electrical_horsepower: float):
        """
        Create a new instance of Power from a value in electrical_horsepower.

        

        :param meters: The Power value in electrical_horsepower.
        :type electrical_horsepower: float
        :return: A new instance of Power.
        :rtype: Power
        """
        return Power(electrical_horsepower, PowerUnits.ElectricalHorsepower)

    
    @staticmethod
    def from_boiler_horsepower(boiler_horsepower: float):
        """
        Create a new instance of Power from a value in boiler_horsepower.

        

        :param meters: The Power value in boiler_horsepower.
        :type boiler_horsepower: float
        :return: A new instance of Power.
        :rtype: Power
        """
        return Power(boiler_horsepower, PowerUnits.BoilerHorsepower)

    
    @staticmethod
    def from_hydraulic_horsepower(hydraulic_horsepower: float):
        """
        Create a new instance of Power from a value in hydraulic_horsepower.

        

        :param meters: The Power value in hydraulic_horsepower.
        :type hydraulic_horsepower: float
        :return: A new instance of Power.
        :rtype: Power
        """
        return Power(hydraulic_horsepower, PowerUnits.HydraulicHorsepower)

    
    @staticmethod
    def from_british_thermal_units_per_hour(british_thermal_units_per_hour: float):
        """
        Create a new instance of Power from a value in british_thermal_units_per_hour.

        

        :param meters: The Power value in british_thermal_units_per_hour.
        :type british_thermal_units_per_hour: float
        :return: A new instance of Power.
        :rtype: Power
        """
        return Power(british_thermal_units_per_hour, PowerUnits.BritishThermalUnitPerHour)

    
    @staticmethod
    def from_joules_per_hour(joules_per_hour: float):
        """
        Create a new instance of Power from a value in joules_per_hour.

        

        :param meters: The Power value in joules_per_hour.
        :type joules_per_hour: float
        :return: A new instance of Power.
        :rtype: Power
        """
        return Power(joules_per_hour, PowerUnits.JoulePerHour)

    
    @staticmethod
    def from_femtowatts(femtowatts: float):
        """
        Create a new instance of Power from a value in femtowatts.

        

        :param meters: The Power value in femtowatts.
        :type femtowatts: float
        :return: A new instance of Power.
        :rtype: Power
        """
        return Power(femtowatts, PowerUnits.Femtowatt)

    
    @staticmethod
    def from_picowatts(picowatts: float):
        """
        Create a new instance of Power from a value in picowatts.

        

        :param meters: The Power value in picowatts.
        :type picowatts: float
        :return: A new instance of Power.
        :rtype: Power
        """
        return Power(picowatts, PowerUnits.Picowatt)

    
    @staticmethod
    def from_nanowatts(nanowatts: float):
        """
        Create a new instance of Power from a value in nanowatts.

        

        :param meters: The Power value in nanowatts.
        :type nanowatts: float
        :return: A new instance of Power.
        :rtype: Power
        """
        return Power(nanowatts, PowerUnits.Nanowatt)

    
    @staticmethod
    def from_microwatts(microwatts: float):
        """
        Create a new instance of Power from a value in microwatts.

        

        :param meters: The Power value in microwatts.
        :type microwatts: float
        :return: A new instance of Power.
        :rtype: Power
        """
        return Power(microwatts, PowerUnits.Microwatt)

    
    @staticmethod
    def from_milliwatts(milliwatts: float):
        """
        Create a new instance of Power from a value in milliwatts.

        

        :param meters: The Power value in milliwatts.
        :type milliwatts: float
        :return: A new instance of Power.
        :rtype: Power
        """
        return Power(milliwatts, PowerUnits.Milliwatt)

    
    @staticmethod
    def from_deciwatts(deciwatts: float):
        """
        Create a new instance of Power from a value in deciwatts.

        

        :param meters: The Power value in deciwatts.
        :type deciwatts: float
        :return: A new instance of Power.
        :rtype: Power
        """
        return Power(deciwatts, PowerUnits.Deciwatt)

    
    @staticmethod
    def from_decawatts(decawatts: float):
        """
        Create a new instance of Power from a value in decawatts.

        

        :param meters: The Power value in decawatts.
        :type decawatts: float
        :return: A new instance of Power.
        :rtype: Power
        """
        return Power(decawatts, PowerUnits.Decawatt)

    
    @staticmethod
    def from_kilowatts(kilowatts: float):
        """
        Create a new instance of Power from a value in kilowatts.

        

        :param meters: The Power value in kilowatts.
        :type kilowatts: float
        :return: A new instance of Power.
        :rtype: Power
        """
        return Power(kilowatts, PowerUnits.Kilowatt)

    
    @staticmethod
    def from_megawatts(megawatts: float):
        """
        Create a new instance of Power from a value in megawatts.

        

        :param meters: The Power value in megawatts.
        :type megawatts: float
        :return: A new instance of Power.
        :rtype: Power
        """
        return Power(megawatts, PowerUnits.Megawatt)

    
    @staticmethod
    def from_gigawatts(gigawatts: float):
        """
        Create a new instance of Power from a value in gigawatts.

        

        :param meters: The Power value in gigawatts.
        :type gigawatts: float
        :return: A new instance of Power.
        :rtype: Power
        """
        return Power(gigawatts, PowerUnits.Gigawatt)

    
    @staticmethod
    def from_terawatts(terawatts: float):
        """
        Create a new instance of Power from a value in terawatts.

        

        :param meters: The Power value in terawatts.
        :type terawatts: float
        :return: A new instance of Power.
        :rtype: Power
        """
        return Power(terawatts, PowerUnits.Terawatt)

    
    @staticmethod
    def from_petawatts(petawatts: float):
        """
        Create a new instance of Power from a value in petawatts.

        

        :param meters: The Power value in petawatts.
        :type petawatts: float
        :return: A new instance of Power.
        :rtype: Power
        """
        return Power(petawatts, PowerUnits.Petawatt)

    
    @staticmethod
    def from_kilobritish_thermal_units_per_hour(kilobritish_thermal_units_per_hour: float):
        """
        Create a new instance of Power from a value in kilobritish_thermal_units_per_hour.

        

        :param meters: The Power value in kilobritish_thermal_units_per_hour.
        :type kilobritish_thermal_units_per_hour: float
        :return: A new instance of Power.
        :rtype: Power
        """
        return Power(kilobritish_thermal_units_per_hour, PowerUnits.KilobritishThermalUnitPerHour)

    
    @staticmethod
    def from_megabritish_thermal_units_per_hour(megabritish_thermal_units_per_hour: float):
        """
        Create a new instance of Power from a value in megabritish_thermal_units_per_hour.

        

        :param meters: The Power value in megabritish_thermal_units_per_hour.
        :type megabritish_thermal_units_per_hour: float
        :return: A new instance of Power.
        :rtype: Power
        """
        return Power(megabritish_thermal_units_per_hour, PowerUnits.MegabritishThermalUnitPerHour)

    
    @staticmethod
    def from_millijoules_per_hour(millijoules_per_hour: float):
        """
        Create a new instance of Power from a value in millijoules_per_hour.

        

        :param meters: The Power value in millijoules_per_hour.
        :type millijoules_per_hour: float
        :return: A new instance of Power.
        :rtype: Power
        """
        return Power(millijoules_per_hour, PowerUnits.MillijoulePerHour)

    
    @staticmethod
    def from_kilojoules_per_hour(kilojoules_per_hour: float):
        """
        Create a new instance of Power from a value in kilojoules_per_hour.

        

        :param meters: The Power value in kilojoules_per_hour.
        :type kilojoules_per_hour: float
        :return: A new instance of Power.
        :rtype: Power
        """
        return Power(kilojoules_per_hour, PowerUnits.KilojoulePerHour)

    
    @staticmethod
    def from_megajoules_per_hour(megajoules_per_hour: float):
        """
        Create a new instance of Power from a value in megajoules_per_hour.

        

        :param meters: The Power value in megajoules_per_hour.
        :type megajoules_per_hour: float
        :return: A new instance of Power.
        :rtype: Power
        """
        return Power(megajoules_per_hour, PowerUnits.MegajoulePerHour)

    
    @staticmethod
    def from_gigajoules_per_hour(gigajoules_per_hour: float):
        """
        Create a new instance of Power from a value in gigajoules_per_hour.

        

        :param meters: The Power value in gigajoules_per_hour.
        :type gigajoules_per_hour: float
        :return: A new instance of Power.
        :rtype: Power
        """
        return Power(gigajoules_per_hour, PowerUnits.GigajoulePerHour)

    
    @property
    def watts(self) -> float:
        """
        
        """
        if self.__watts != None:
            return self.__watts
        self.__watts = self.__convert_from_base(PowerUnits.Watt)
        return self.__watts

    
    @property
    def mechanical_horsepower(self) -> float:
        """
        
        """
        if self.__mechanical_horsepower != None:
            return self.__mechanical_horsepower
        self.__mechanical_horsepower = self.__convert_from_base(PowerUnits.MechanicalHorsepower)
        return self.__mechanical_horsepower

    
    @property
    def metric_horsepower(self) -> float:
        """
        
        """
        if self.__metric_horsepower != None:
            return self.__metric_horsepower
        self.__metric_horsepower = self.__convert_from_base(PowerUnits.MetricHorsepower)
        return self.__metric_horsepower

    
    @property
    def electrical_horsepower(self) -> float:
        """
        
        """
        if self.__electrical_horsepower != None:
            return self.__electrical_horsepower
        self.__electrical_horsepower = self.__convert_from_base(PowerUnits.ElectricalHorsepower)
        return self.__electrical_horsepower

    
    @property
    def boiler_horsepower(self) -> float:
        """
        
        """
        if self.__boiler_horsepower != None:
            return self.__boiler_horsepower
        self.__boiler_horsepower = self.__convert_from_base(PowerUnits.BoilerHorsepower)
        return self.__boiler_horsepower

    
    @property
    def hydraulic_horsepower(self) -> float:
        """
        
        """
        if self.__hydraulic_horsepower != None:
            return self.__hydraulic_horsepower
        self.__hydraulic_horsepower = self.__convert_from_base(PowerUnits.HydraulicHorsepower)
        return self.__hydraulic_horsepower

    
    @property
    def british_thermal_units_per_hour(self) -> float:
        """
        
        """
        if self.__british_thermal_units_per_hour != None:
            return self.__british_thermal_units_per_hour
        self.__british_thermal_units_per_hour = self.__convert_from_base(PowerUnits.BritishThermalUnitPerHour)
        return self.__british_thermal_units_per_hour

    
    @property
    def joules_per_hour(self) -> float:
        """
        
        """
        if self.__joules_per_hour != None:
            return self.__joules_per_hour
        self.__joules_per_hour = self.__convert_from_base(PowerUnits.JoulePerHour)
        return self.__joules_per_hour

    
    @property
    def femtowatts(self) -> float:
        """
        
        """
        if self.__femtowatts != None:
            return self.__femtowatts
        self.__femtowatts = self.__convert_from_base(PowerUnits.Femtowatt)
        return self.__femtowatts

    
    @property
    def picowatts(self) -> float:
        """
        
        """
        if self.__picowatts != None:
            return self.__picowatts
        self.__picowatts = self.__convert_from_base(PowerUnits.Picowatt)
        return self.__picowatts

    
    @property
    def nanowatts(self) -> float:
        """
        
        """
        if self.__nanowatts != None:
            return self.__nanowatts
        self.__nanowatts = self.__convert_from_base(PowerUnits.Nanowatt)
        return self.__nanowatts

    
    @property
    def microwatts(self) -> float:
        """
        
        """
        if self.__microwatts != None:
            return self.__microwatts
        self.__microwatts = self.__convert_from_base(PowerUnits.Microwatt)
        return self.__microwatts

    
    @property
    def milliwatts(self) -> float:
        """
        
        """
        if self.__milliwatts != None:
            return self.__milliwatts
        self.__milliwatts = self.__convert_from_base(PowerUnits.Milliwatt)
        return self.__milliwatts

    
    @property
    def deciwatts(self) -> float:
        """
        
        """
        if self.__deciwatts != None:
            return self.__deciwatts
        self.__deciwatts = self.__convert_from_base(PowerUnits.Deciwatt)
        return self.__deciwatts

    
    @property
    def decawatts(self) -> float:
        """
        
        """
        if self.__decawatts != None:
            return self.__decawatts
        self.__decawatts = self.__convert_from_base(PowerUnits.Decawatt)
        return self.__decawatts

    
    @property
    def kilowatts(self) -> float:
        """
        
        """
        if self.__kilowatts != None:
            return self.__kilowatts
        self.__kilowatts = self.__convert_from_base(PowerUnits.Kilowatt)
        return self.__kilowatts

    
    @property
    def megawatts(self) -> float:
        """
        
        """
        if self.__megawatts != None:
            return self.__megawatts
        self.__megawatts = self.__convert_from_base(PowerUnits.Megawatt)
        return self.__megawatts

    
    @property
    def gigawatts(self) -> float:
        """
        
        """
        if self.__gigawatts != None:
            return self.__gigawatts
        self.__gigawatts = self.__convert_from_base(PowerUnits.Gigawatt)
        return self.__gigawatts

    
    @property
    def terawatts(self) -> float:
        """
        
        """
        if self.__terawatts != None:
            return self.__terawatts
        self.__terawatts = self.__convert_from_base(PowerUnits.Terawatt)
        return self.__terawatts

    
    @property
    def petawatts(self) -> float:
        """
        
        """
        if self.__petawatts != None:
            return self.__petawatts
        self.__petawatts = self.__convert_from_base(PowerUnits.Petawatt)
        return self.__petawatts

    
    @property
    def kilobritish_thermal_units_per_hour(self) -> float:
        """
        
        """
        if self.__kilobritish_thermal_units_per_hour != None:
            return self.__kilobritish_thermal_units_per_hour
        self.__kilobritish_thermal_units_per_hour = self.__convert_from_base(PowerUnits.KilobritishThermalUnitPerHour)
        return self.__kilobritish_thermal_units_per_hour

    
    @property
    def megabritish_thermal_units_per_hour(self) -> float:
        """
        
        """
        if self.__megabritish_thermal_units_per_hour != None:
            return self.__megabritish_thermal_units_per_hour
        self.__megabritish_thermal_units_per_hour = self.__convert_from_base(PowerUnits.MegabritishThermalUnitPerHour)
        return self.__megabritish_thermal_units_per_hour

    
    @property
    def millijoules_per_hour(self) -> float:
        """
        
        """
        if self.__millijoules_per_hour != None:
            return self.__millijoules_per_hour
        self.__millijoules_per_hour = self.__convert_from_base(PowerUnits.MillijoulePerHour)
        return self.__millijoules_per_hour

    
    @property
    def kilojoules_per_hour(self) -> float:
        """
        
        """
        if self.__kilojoules_per_hour != None:
            return self.__kilojoules_per_hour
        self.__kilojoules_per_hour = self.__convert_from_base(PowerUnits.KilojoulePerHour)
        return self.__kilojoules_per_hour

    
    @property
    def megajoules_per_hour(self) -> float:
        """
        
        """
        if self.__megajoules_per_hour != None:
            return self.__megajoules_per_hour
        self.__megajoules_per_hour = self.__convert_from_base(PowerUnits.MegajoulePerHour)
        return self.__megajoules_per_hour

    
    @property
    def gigajoules_per_hour(self) -> float:
        """
        
        """
        if self.__gigajoules_per_hour != None:
            return self.__gigajoules_per_hour
        self.__gigajoules_per_hour = self.__convert_from_base(PowerUnits.GigajoulePerHour)
        return self.__gigajoules_per_hour

    
    def to_string(self, unit: PowerUnits = PowerUnits.Watt) -> string:
        """
        Format the Power to string.
        Note! the default format for Power is Watt.
        To specify the unit format set the 'unit' parameter.
        """
        
        if unit == PowerUnits.Watt:
            return f"""{self.watts} W"""
        
        if unit == PowerUnits.MechanicalHorsepower:
            return f"""{self.mechanical_horsepower} hp(I)"""
        
        if unit == PowerUnits.MetricHorsepower:
            return f"""{self.metric_horsepower} hp(M)"""
        
        if unit == PowerUnits.ElectricalHorsepower:
            return f"""{self.electrical_horsepower} hp(E)"""
        
        if unit == PowerUnits.BoilerHorsepower:
            return f"""{self.boiler_horsepower} hp(S)"""
        
        if unit == PowerUnits.HydraulicHorsepower:
            return f"""{self.hydraulic_horsepower} hp(H)"""
        
        if unit == PowerUnits.BritishThermalUnitPerHour:
            return f"""{self.british_thermal_units_per_hour} Btu/h"""
        
        if unit == PowerUnits.JoulePerHour:
            return f"""{self.joules_per_hour} J/h"""
        
        if unit == PowerUnits.Femtowatt:
            return f"""{self.femtowatts} """
        
        if unit == PowerUnits.Picowatt:
            return f"""{self.picowatts} """
        
        if unit == PowerUnits.Nanowatt:
            return f"""{self.nanowatts} """
        
        if unit == PowerUnits.Microwatt:
            return f"""{self.microwatts} """
        
        if unit == PowerUnits.Milliwatt:
            return f"""{self.milliwatts} """
        
        if unit == PowerUnits.Deciwatt:
            return f"""{self.deciwatts} """
        
        if unit == PowerUnits.Decawatt:
            return f"""{self.decawatts} """
        
        if unit == PowerUnits.Kilowatt:
            return f"""{self.kilowatts} """
        
        if unit == PowerUnits.Megawatt:
            return f"""{self.megawatts} """
        
        if unit == PowerUnits.Gigawatt:
            return f"""{self.gigawatts} """
        
        if unit == PowerUnits.Terawatt:
            return f"""{self.terawatts} """
        
        if unit == PowerUnits.Petawatt:
            return f"""{self.petawatts} """
        
        if unit == PowerUnits.KilobritishThermalUnitPerHour:
            return f"""{self.kilobritish_thermal_units_per_hour} """
        
        if unit == PowerUnits.MegabritishThermalUnitPerHour:
            return f"""{self.megabritish_thermal_units_per_hour} """
        
        if unit == PowerUnits.MillijoulePerHour:
            return f"""{self.millijoules_per_hour} """
        
        if unit == PowerUnits.KilojoulePerHour:
            return f"""{self.kilojoules_per_hour} """
        
        if unit == PowerUnits.MegajoulePerHour:
            return f"""{self.megajoules_per_hour} """
        
        if unit == PowerUnits.GigajoulePerHour:
            return f"""{self.gigajoules_per_hour} """
        
        return f'{self.__value}'


    def get_unit_abbreviation(self, unit_abbreviation: PowerUnits = PowerUnits.Watt) -> string:
        """
        Get Power unit abbreviation.
        Note! the default abbreviation for Power is Watt.
        To specify the unit abbreviation set the 'unit_abbreviation' parameter.
        """
        
        if unit_abbreviation == PowerUnits.Watt:
            return """W"""
        
        if unit_abbreviation == PowerUnits.MechanicalHorsepower:
            return """hp(I)"""
        
        if unit_abbreviation == PowerUnits.MetricHorsepower:
            return """hp(M)"""
        
        if unit_abbreviation == PowerUnits.ElectricalHorsepower:
            return """hp(E)"""
        
        if unit_abbreviation == PowerUnits.BoilerHorsepower:
            return """hp(S)"""
        
        if unit_abbreviation == PowerUnits.HydraulicHorsepower:
            return """hp(H)"""
        
        if unit_abbreviation == PowerUnits.BritishThermalUnitPerHour:
            return """Btu/h"""
        
        if unit_abbreviation == PowerUnits.JoulePerHour:
            return """J/h"""
        
        if unit_abbreviation == PowerUnits.Femtowatt:
            return """"""
        
        if unit_abbreviation == PowerUnits.Picowatt:
            return """"""
        
        if unit_abbreviation == PowerUnits.Nanowatt:
            return """"""
        
        if unit_abbreviation == PowerUnits.Microwatt:
            return """"""
        
        if unit_abbreviation == PowerUnits.Milliwatt:
            return """"""
        
        if unit_abbreviation == PowerUnits.Deciwatt:
            return """"""
        
        if unit_abbreviation == PowerUnits.Decawatt:
            return """"""
        
        if unit_abbreviation == PowerUnits.Kilowatt:
            return """"""
        
        if unit_abbreviation == PowerUnits.Megawatt:
            return """"""
        
        if unit_abbreviation == PowerUnits.Gigawatt:
            return """"""
        
        if unit_abbreviation == PowerUnits.Terawatt:
            return """"""
        
        if unit_abbreviation == PowerUnits.Petawatt:
            return """"""
        
        if unit_abbreviation == PowerUnits.KilobritishThermalUnitPerHour:
            return """"""
        
        if unit_abbreviation == PowerUnits.MegabritishThermalUnitPerHour:
            return """"""
        
        if unit_abbreviation == PowerUnits.MillijoulePerHour:
            return """"""
        
        if unit_abbreviation == PowerUnits.KilojoulePerHour:
            return """"""
        
        if unit_abbreviation == PowerUnits.MegajoulePerHour:
            return """"""
        
        if unit_abbreviation == PowerUnits.GigajoulePerHour:
            return """"""
        

    def __str__(self):
        return self.to_string()


    def __add__(self, other):
        if not isinstance(other, Power):
            raise TypeError("unsupported operand type(s) for +: 'Power' and '{}'".format(type(other).__name__))
        return Power(self.__value + other.__value)


    def __mul__(self, other):
        if not isinstance(other, Power):
            raise TypeError("unsupported operand type(s) for *: 'Power' and '{}'".format(type(other).__name__))
        return Power(self.__value * other.__value)


    def __sub__(self, other):
        if not isinstance(other, Power):
            raise TypeError("unsupported operand type(s) for -: 'Power' and '{}'".format(type(other).__name__))
        return Power(self.__value - other.__value)


    def __truediv__(self, other):
        if not isinstance(other, Power):
            raise TypeError("unsupported operand type(s) for /: 'Power' and '{}'".format(type(other).__name__))
        return Power(self.__value / other.__value)


    def __mod__(self, other):
        if not isinstance(other, Power):
            raise TypeError("unsupported operand type(s) for %: 'Power' and '{}'".format(type(other).__name__))
        return Power(self.__value % other.__value)


    def __pow__(self, other):
        if not isinstance(other, Power):
            raise TypeError("unsupported operand type(s) for **: 'Power' and '{}'".format(type(other).__name__))
        return Power(self.__value ** other.__value)


    def __eq__(self, other):
        if not isinstance(other, Power):
            raise TypeError("unsupported operand type(s) for ==: 'Power' and '{}'".format(type(other).__name__))
        return self.__value == other.__value


    def __lt__(self, other):
        if not isinstance(other, Power):
            raise TypeError("unsupported operand type(s) for <: 'Power' and '{}'".format(type(other).__name__))
        return self.__value < other.__value


    def __gt__(self, other):
        if not isinstance(other, Power):
            raise TypeError("unsupported operand type(s) for >: 'Power' and '{}'".format(type(other).__name__))
        return self.__value > other.__value


    def __le__(self, other):
        if not isinstance(other, Power):
            raise TypeError("unsupported operand type(s) for <=: 'Power' and '{}'".format(type(other).__name__))
        return self.__value <= other.__value


    def __ge__(self, other):
        if not isinstance(other, Power):
            raise TypeError("unsupported operand type(s) for >=: 'Power' and '{}'".format(type(other).__name__))
        return self.__value >= other.__value


from enum import Enum
import math
import string


class SpecificEntropyUnits(Enum):
        """
            SpecificEntropyUnits enumeration
        """
        
        JoulePerKilogramKelvin = 'joule_per_kilogram_kelvin'
        """
            
        """
        
        JoulePerKilogramDegreeCelsius = 'joule_per_kilogram_degree_celsius'
        """
            
        """
        
        CaloriePerGramKelvin = 'calorie_per_gram_kelvin'
        """
            
        """
        
        BtuPerPoundFahrenheit = 'btu_per_pound_fahrenheit'
        """
            
        """
        
        KilojoulePerKilogramKelvin = 'kilojoule_per_kilogram_kelvin'
        """
            
        """
        
        MegajoulePerKilogramKelvin = 'megajoule_per_kilogram_kelvin'
        """
            
        """
        
        KilojoulePerKilogramDegreeCelsius = 'kilojoule_per_kilogram_degree_celsius'
        """
            
        """
        
        MegajoulePerKilogramDegreeCelsius = 'megajoule_per_kilogram_degree_celsius'
        """
            
        """
        
        KilocaloriePerGramKelvin = 'kilocalorie_per_gram_kelvin'
        """
            
        """
        

class SpecificEntropy:
    """
    Specific entropy is an amount of energy required to raise temperature of a substance by 1 Kelvin per unit mass.

    Args:
        value (float): The value.
        from_unit (SpecificEntropyUnits): The SpecificEntropy unit to create from, The default unit is JoulePerKilogramKelvin
    """
    def __init__(self, value: float, from_unit: SpecificEntropyUnits = SpecificEntropyUnits.JoulePerKilogramKelvin):
        if math.isnan(value):
            raise ValueError('Invalid unit: value is NaN')
        self.__value = self.__convert_to_base(value, from_unit)
        
        self.__joules_per_kilogram_kelvin = None
        
        self.__joules_per_kilogram_degree_celsius = None
        
        self.__calories_per_gram_kelvin = None
        
        self.__btus_per_pound_fahrenheit = None
        
        self.__kilojoules_per_kilogram_kelvin = None
        
        self.__megajoules_per_kilogram_kelvin = None
        
        self.__kilojoules_per_kilogram_degree_celsius = None
        
        self.__megajoules_per_kilogram_degree_celsius = None
        
        self.__kilocalories_per_gram_kelvin = None
        

    def __convert_from_base(self, from_unit: SpecificEntropyUnits) -> float:
        value = self.__value
        
        if from_unit == SpecificEntropyUnits.JoulePerKilogramKelvin:
            return (value)
        
        if from_unit == SpecificEntropyUnits.JoulePerKilogramDegreeCelsius:
            return (value)
        
        if from_unit == SpecificEntropyUnits.CaloriePerGramKelvin:
            return (value / 4.184e3)
        
        if from_unit == SpecificEntropyUnits.BtuPerPoundFahrenheit:
            return (value / 4.1868e3)
        
        if from_unit == SpecificEntropyUnits.KilojoulePerKilogramKelvin:
            return ((value) / 1000.0)
        
        if from_unit == SpecificEntropyUnits.MegajoulePerKilogramKelvin:
            return ((value) / 1000000.0)
        
        if from_unit == SpecificEntropyUnits.KilojoulePerKilogramDegreeCelsius:
            return ((value) / 1000.0)
        
        if from_unit == SpecificEntropyUnits.MegajoulePerKilogramDegreeCelsius:
            return ((value) / 1000000.0)
        
        if from_unit == SpecificEntropyUnits.KilocaloriePerGramKelvin:
            return ((value / 4.184e3) / 1000.0)
        
        return None


    def __convert_to_base(self, value: float, to_unit: SpecificEntropyUnits) -> float:
        
        if to_unit == SpecificEntropyUnits.JoulePerKilogramKelvin:
            return (value)
        
        if to_unit == SpecificEntropyUnits.JoulePerKilogramDegreeCelsius:
            return (value)
        
        if to_unit == SpecificEntropyUnits.CaloriePerGramKelvin:
            return (value * 4.184e3)
        
        if to_unit == SpecificEntropyUnits.BtuPerPoundFahrenheit:
            return (value * 4.1868e3)
        
        if to_unit == SpecificEntropyUnits.KilojoulePerKilogramKelvin:
            return ((value) * 1000.0)
        
        if to_unit == SpecificEntropyUnits.MegajoulePerKilogramKelvin:
            return ((value) * 1000000.0)
        
        if to_unit == SpecificEntropyUnits.KilojoulePerKilogramDegreeCelsius:
            return ((value) * 1000.0)
        
        if to_unit == SpecificEntropyUnits.MegajoulePerKilogramDegreeCelsius:
            return ((value) * 1000000.0)
        
        if to_unit == SpecificEntropyUnits.KilocaloriePerGramKelvin:
            return ((value * 4.184e3) * 1000.0)
        
        return None


    @property
    def base_value(self) -> float:
        return self.__value

    
    @staticmethod
    def from_joules_per_kilogram_kelvin(joules_per_kilogram_kelvin: float):
        """
        Create a new instance of SpecificEntropy from a value in joules_per_kilogram_kelvin.

        

        :param meters: The SpecificEntropy value in joules_per_kilogram_kelvin.
        :type joules_per_kilogram_kelvin: float
        :return: A new instance of SpecificEntropy.
        :rtype: SpecificEntropy
        """
        return SpecificEntropy(joules_per_kilogram_kelvin, SpecificEntropyUnits.JoulePerKilogramKelvin)

    
    @staticmethod
    def from_joules_per_kilogram_degree_celsius(joules_per_kilogram_degree_celsius: float):
        """
        Create a new instance of SpecificEntropy from a value in joules_per_kilogram_degree_celsius.

        

        :param meters: The SpecificEntropy value in joules_per_kilogram_degree_celsius.
        :type joules_per_kilogram_degree_celsius: float
        :return: A new instance of SpecificEntropy.
        :rtype: SpecificEntropy
        """
        return SpecificEntropy(joules_per_kilogram_degree_celsius, SpecificEntropyUnits.JoulePerKilogramDegreeCelsius)

    
    @staticmethod
    def from_calories_per_gram_kelvin(calories_per_gram_kelvin: float):
        """
        Create a new instance of SpecificEntropy from a value in calories_per_gram_kelvin.

        

        :param meters: The SpecificEntropy value in calories_per_gram_kelvin.
        :type calories_per_gram_kelvin: float
        :return: A new instance of SpecificEntropy.
        :rtype: SpecificEntropy
        """
        return SpecificEntropy(calories_per_gram_kelvin, SpecificEntropyUnits.CaloriePerGramKelvin)

    
    @staticmethod
    def from_btus_per_pound_fahrenheit(btus_per_pound_fahrenheit: float):
        """
        Create a new instance of SpecificEntropy from a value in btus_per_pound_fahrenheit.

        

        :param meters: The SpecificEntropy value in btus_per_pound_fahrenheit.
        :type btus_per_pound_fahrenheit: float
        :return: A new instance of SpecificEntropy.
        :rtype: SpecificEntropy
        """
        return SpecificEntropy(btus_per_pound_fahrenheit, SpecificEntropyUnits.BtuPerPoundFahrenheit)

    
    @staticmethod
    def from_kilojoules_per_kilogram_kelvin(kilojoules_per_kilogram_kelvin: float):
        """
        Create a new instance of SpecificEntropy from a value in kilojoules_per_kilogram_kelvin.

        

        :param meters: The SpecificEntropy value in kilojoules_per_kilogram_kelvin.
        :type kilojoules_per_kilogram_kelvin: float
        :return: A new instance of SpecificEntropy.
        :rtype: SpecificEntropy
        """
        return SpecificEntropy(kilojoules_per_kilogram_kelvin, SpecificEntropyUnits.KilojoulePerKilogramKelvin)

    
    @staticmethod
    def from_megajoules_per_kilogram_kelvin(megajoules_per_kilogram_kelvin: float):
        """
        Create a new instance of SpecificEntropy from a value in megajoules_per_kilogram_kelvin.

        

        :param meters: The SpecificEntropy value in megajoules_per_kilogram_kelvin.
        :type megajoules_per_kilogram_kelvin: float
        :return: A new instance of SpecificEntropy.
        :rtype: SpecificEntropy
        """
        return SpecificEntropy(megajoules_per_kilogram_kelvin, SpecificEntropyUnits.MegajoulePerKilogramKelvin)

    
    @staticmethod
    def from_kilojoules_per_kilogram_degree_celsius(kilojoules_per_kilogram_degree_celsius: float):
        """
        Create a new instance of SpecificEntropy from a value in kilojoules_per_kilogram_degree_celsius.

        

        :param meters: The SpecificEntropy value in kilojoules_per_kilogram_degree_celsius.
        :type kilojoules_per_kilogram_degree_celsius: float
        :return: A new instance of SpecificEntropy.
        :rtype: SpecificEntropy
        """
        return SpecificEntropy(kilojoules_per_kilogram_degree_celsius, SpecificEntropyUnits.KilojoulePerKilogramDegreeCelsius)

    
    @staticmethod
    def from_megajoules_per_kilogram_degree_celsius(megajoules_per_kilogram_degree_celsius: float):
        """
        Create a new instance of SpecificEntropy from a value in megajoules_per_kilogram_degree_celsius.

        

        :param meters: The SpecificEntropy value in megajoules_per_kilogram_degree_celsius.
        :type megajoules_per_kilogram_degree_celsius: float
        :return: A new instance of SpecificEntropy.
        :rtype: SpecificEntropy
        """
        return SpecificEntropy(megajoules_per_kilogram_degree_celsius, SpecificEntropyUnits.MegajoulePerKilogramDegreeCelsius)

    
    @staticmethod
    def from_kilocalories_per_gram_kelvin(kilocalories_per_gram_kelvin: float):
        """
        Create a new instance of SpecificEntropy from a value in kilocalories_per_gram_kelvin.

        

        :param meters: The SpecificEntropy value in kilocalories_per_gram_kelvin.
        :type kilocalories_per_gram_kelvin: float
        :return: A new instance of SpecificEntropy.
        :rtype: SpecificEntropy
        """
        return SpecificEntropy(kilocalories_per_gram_kelvin, SpecificEntropyUnits.KilocaloriePerGramKelvin)

    
    @property
    def joules_per_kilogram_kelvin(self) -> float:
        """
        
        """
        if self.__joules_per_kilogram_kelvin != None:
            return self.__joules_per_kilogram_kelvin
        self.__joules_per_kilogram_kelvin = self.__convert_from_base(SpecificEntropyUnits.JoulePerKilogramKelvin)
        return self.__joules_per_kilogram_kelvin

    
    @property
    def joules_per_kilogram_degree_celsius(self) -> float:
        """
        
        """
        if self.__joules_per_kilogram_degree_celsius != None:
            return self.__joules_per_kilogram_degree_celsius
        self.__joules_per_kilogram_degree_celsius = self.__convert_from_base(SpecificEntropyUnits.JoulePerKilogramDegreeCelsius)
        return self.__joules_per_kilogram_degree_celsius

    
    @property
    def calories_per_gram_kelvin(self) -> float:
        """
        
        """
        if self.__calories_per_gram_kelvin != None:
            return self.__calories_per_gram_kelvin
        self.__calories_per_gram_kelvin = self.__convert_from_base(SpecificEntropyUnits.CaloriePerGramKelvin)
        return self.__calories_per_gram_kelvin

    
    @property
    def btus_per_pound_fahrenheit(self) -> float:
        """
        
        """
        if self.__btus_per_pound_fahrenheit != None:
            return self.__btus_per_pound_fahrenheit
        self.__btus_per_pound_fahrenheit = self.__convert_from_base(SpecificEntropyUnits.BtuPerPoundFahrenheit)
        return self.__btus_per_pound_fahrenheit

    
    @property
    def kilojoules_per_kilogram_kelvin(self) -> float:
        """
        
        """
        if self.__kilojoules_per_kilogram_kelvin != None:
            return self.__kilojoules_per_kilogram_kelvin
        self.__kilojoules_per_kilogram_kelvin = self.__convert_from_base(SpecificEntropyUnits.KilojoulePerKilogramKelvin)
        return self.__kilojoules_per_kilogram_kelvin

    
    @property
    def megajoules_per_kilogram_kelvin(self) -> float:
        """
        
        """
        if self.__megajoules_per_kilogram_kelvin != None:
            return self.__megajoules_per_kilogram_kelvin
        self.__megajoules_per_kilogram_kelvin = self.__convert_from_base(SpecificEntropyUnits.MegajoulePerKilogramKelvin)
        return self.__megajoules_per_kilogram_kelvin

    
    @property
    def kilojoules_per_kilogram_degree_celsius(self) -> float:
        """
        
        """
        if self.__kilojoules_per_kilogram_degree_celsius != None:
            return self.__kilojoules_per_kilogram_degree_celsius
        self.__kilojoules_per_kilogram_degree_celsius = self.__convert_from_base(SpecificEntropyUnits.KilojoulePerKilogramDegreeCelsius)
        return self.__kilojoules_per_kilogram_degree_celsius

    
    @property
    def megajoules_per_kilogram_degree_celsius(self) -> float:
        """
        
        """
        if self.__megajoules_per_kilogram_degree_celsius != None:
            return self.__megajoules_per_kilogram_degree_celsius
        self.__megajoules_per_kilogram_degree_celsius = self.__convert_from_base(SpecificEntropyUnits.MegajoulePerKilogramDegreeCelsius)
        return self.__megajoules_per_kilogram_degree_celsius

    
    @property
    def kilocalories_per_gram_kelvin(self) -> float:
        """
        
        """
        if self.__kilocalories_per_gram_kelvin != None:
            return self.__kilocalories_per_gram_kelvin
        self.__kilocalories_per_gram_kelvin = self.__convert_from_base(SpecificEntropyUnits.KilocaloriePerGramKelvin)
        return self.__kilocalories_per_gram_kelvin

    
    def to_string(self, unit: SpecificEntropyUnits = SpecificEntropyUnits.JoulePerKilogramKelvin) -> string:
        """
        Format the SpecificEntropy to string.
        Note! the default format for SpecificEntropy is JoulePerKilogramKelvin.
        To specify the unit format set the 'unit' parameter.
        """
        
        if unit == SpecificEntropyUnits.JoulePerKilogramKelvin:
            return f"""{self.joules_per_kilogram_kelvin} J/kg.K"""
        
        if unit == SpecificEntropyUnits.JoulePerKilogramDegreeCelsius:
            return f"""{self.joules_per_kilogram_degree_celsius} J/kg.C"""
        
        if unit == SpecificEntropyUnits.CaloriePerGramKelvin:
            return f"""{self.calories_per_gram_kelvin} cal/g.K"""
        
        if unit == SpecificEntropyUnits.BtuPerPoundFahrenheit:
            return f"""{self.btus_per_pound_fahrenheit} BTU/lb·°F"""
        
        if unit == SpecificEntropyUnits.KilojoulePerKilogramKelvin:
            return f"""{self.kilojoules_per_kilogram_kelvin} """
        
        if unit == SpecificEntropyUnits.MegajoulePerKilogramKelvin:
            return f"""{self.megajoules_per_kilogram_kelvin} """
        
        if unit == SpecificEntropyUnits.KilojoulePerKilogramDegreeCelsius:
            return f"""{self.kilojoules_per_kilogram_degree_celsius} """
        
        if unit == SpecificEntropyUnits.MegajoulePerKilogramDegreeCelsius:
            return f"""{self.megajoules_per_kilogram_degree_celsius} """
        
        if unit == SpecificEntropyUnits.KilocaloriePerGramKelvin:
            return f"""{self.kilocalories_per_gram_kelvin} """
        
        return f'{self.__value}'


    def get_unit_abbreviation(self, unit_abbreviation: SpecificEntropyUnits = SpecificEntropyUnits.JoulePerKilogramKelvin) -> string:
        """
        Get SpecificEntropy unit abbreviation.
        Note! the default abbreviation for SpecificEntropy is JoulePerKilogramKelvin.
        To specify the unit abbreviation set the 'unit_abbreviation' parameter.
        """
        
        if unit_abbreviation == SpecificEntropyUnits.JoulePerKilogramKelvin:
            return """J/kg.K"""
        
        if unit_abbreviation == SpecificEntropyUnits.JoulePerKilogramDegreeCelsius:
            return """J/kg.C"""
        
        if unit_abbreviation == SpecificEntropyUnits.CaloriePerGramKelvin:
            return """cal/g.K"""
        
        if unit_abbreviation == SpecificEntropyUnits.BtuPerPoundFahrenheit:
            return """BTU/lb·°F"""
        
        if unit_abbreviation == SpecificEntropyUnits.KilojoulePerKilogramKelvin:
            return """"""
        
        if unit_abbreviation == SpecificEntropyUnits.MegajoulePerKilogramKelvin:
            return """"""
        
        if unit_abbreviation == SpecificEntropyUnits.KilojoulePerKilogramDegreeCelsius:
            return """"""
        
        if unit_abbreviation == SpecificEntropyUnits.MegajoulePerKilogramDegreeCelsius:
            return """"""
        
        if unit_abbreviation == SpecificEntropyUnits.KilocaloriePerGramKelvin:
            return """"""
        

    def __str__(self):
        return self.to_string()


    def __add__(self, other):
        if not isinstance(other, SpecificEntropy):
            raise TypeError("unsupported operand type(s) for +: 'SpecificEntropy' and '{}'".format(type(other).__name__))
        return SpecificEntropy(self.__value + other.__value)


    def __mul__(self, other):
        if not isinstance(other, SpecificEntropy):
            raise TypeError("unsupported operand type(s) for *: 'SpecificEntropy' and '{}'".format(type(other).__name__))
        return SpecificEntropy(self.__value * other.__value)


    def __sub__(self, other):
        if not isinstance(other, SpecificEntropy):
            raise TypeError("unsupported operand type(s) for -: 'SpecificEntropy' and '{}'".format(type(other).__name__))
        return SpecificEntropy(self.__value - other.__value)


    def __truediv__(self, other):
        if not isinstance(other, SpecificEntropy):
            raise TypeError("unsupported operand type(s) for /: 'SpecificEntropy' and '{}'".format(type(other).__name__))
        return SpecificEntropy(self.__value / other.__value)


    def __mod__(self, other):
        if not isinstance(other, SpecificEntropy):
            raise TypeError("unsupported operand type(s) for %: 'SpecificEntropy' and '{}'".format(type(other).__name__))
        return SpecificEntropy(self.__value % other.__value)


    def __pow__(self, other):
        if not isinstance(other, SpecificEntropy):
            raise TypeError("unsupported operand type(s) for **: 'SpecificEntropy' and '{}'".format(type(other).__name__))
        return SpecificEntropy(self.__value ** other.__value)


    def __eq__(self, other):
        if not isinstance(other, SpecificEntropy):
            raise TypeError("unsupported operand type(s) for ==: 'SpecificEntropy' and '{}'".format(type(other).__name__))
        return self.__value == other.__value


    def __lt__(self, other):
        if not isinstance(other, SpecificEntropy):
            raise TypeError("unsupported operand type(s) for <: 'SpecificEntropy' and '{}'".format(type(other).__name__))
        return self.__value < other.__value


    def __gt__(self, other):
        if not isinstance(other, SpecificEntropy):
            raise TypeError("unsupported operand type(s) for >: 'SpecificEntropy' and '{}'".format(type(other).__name__))
        return self.__value > other.__value


    def __le__(self, other):
        if not isinstance(other, SpecificEntropy):
            raise TypeError("unsupported operand type(s) for <=: 'SpecificEntropy' and '{}'".format(type(other).__name__))
        return self.__value <= other.__value


    def __ge__(self, other):
        if not isinstance(other, SpecificEntropy):
            raise TypeError("unsupported operand type(s) for >=: 'SpecificEntropy' and '{}'".format(type(other).__name__))
        return self.__value >= other.__value


from enum import Enum
import math
import string


class RatioChangeRateUnits(Enum):
        """
            RatioChangeRateUnits enumeration
        """
        
        PercentPerSecond = 'percent_per_second'
        """
            
        """
        
        DecimalFractionPerSecond = 'decimal_fraction_per_second'
        """
            
        """
        

class RatioChangeRate:
    """
    The change in ratio per unit of time.

    Args:
        value (float): The value.
        from_unit (RatioChangeRateUnits): The RatioChangeRate unit to create from, The default unit is DecimalFractionPerSecond
    """
    def __init__(self, value: float, from_unit: RatioChangeRateUnits = RatioChangeRateUnits.DecimalFractionPerSecond):
        if math.isnan(value):
            raise ValueError('Invalid unit: value is NaN')
        self.__value = self.__convert_to_base(value, from_unit)
        
        self.__percents_per_second = None
        
        self.__decimal_fractions_per_second = None
        

    def __convert_from_base(self, from_unit: RatioChangeRateUnits) -> float:
        value = self.__value
        
        if from_unit == RatioChangeRateUnits.PercentPerSecond:
            return (value * 1e2)
        
        if from_unit == RatioChangeRateUnits.DecimalFractionPerSecond:
            return (value)
        
        return None


    def __convert_to_base(self, value: float, to_unit: RatioChangeRateUnits) -> float:
        
        if to_unit == RatioChangeRateUnits.PercentPerSecond:
            return (value / 1e2)
        
        if to_unit == RatioChangeRateUnits.DecimalFractionPerSecond:
            return (value)
        
        return None


    @property
    def base_value(self) -> float:
        return self.__value

    
    @staticmethod
    def from_percents_per_second(percents_per_second: float):
        """
        Create a new instance of RatioChangeRate from a value in percents_per_second.

        

        :param meters: The RatioChangeRate value in percents_per_second.
        :type percents_per_second: float
        :return: A new instance of RatioChangeRate.
        :rtype: RatioChangeRate
        """
        return RatioChangeRate(percents_per_second, RatioChangeRateUnits.PercentPerSecond)

    
    @staticmethod
    def from_decimal_fractions_per_second(decimal_fractions_per_second: float):
        """
        Create a new instance of RatioChangeRate from a value in decimal_fractions_per_second.

        

        :param meters: The RatioChangeRate value in decimal_fractions_per_second.
        :type decimal_fractions_per_second: float
        :return: A new instance of RatioChangeRate.
        :rtype: RatioChangeRate
        """
        return RatioChangeRate(decimal_fractions_per_second, RatioChangeRateUnits.DecimalFractionPerSecond)

    
    @property
    def percents_per_second(self) -> float:
        """
        
        """
        if self.__percents_per_second != None:
            return self.__percents_per_second
        self.__percents_per_second = self.__convert_from_base(RatioChangeRateUnits.PercentPerSecond)
        return self.__percents_per_second

    
    @property
    def decimal_fractions_per_second(self) -> float:
        """
        
        """
        if self.__decimal_fractions_per_second != None:
            return self.__decimal_fractions_per_second
        self.__decimal_fractions_per_second = self.__convert_from_base(RatioChangeRateUnits.DecimalFractionPerSecond)
        return self.__decimal_fractions_per_second

    
    def to_string(self, unit: RatioChangeRateUnits = RatioChangeRateUnits.DecimalFractionPerSecond) -> string:
        """
        Format the RatioChangeRate to string.
        Note! the default format for RatioChangeRate is DecimalFractionPerSecond.
        To specify the unit format set the 'unit' parameter.
        """
        
        if unit == RatioChangeRateUnits.PercentPerSecond:
            return f"""{self.percents_per_second} %/s"""
        
        if unit == RatioChangeRateUnits.DecimalFractionPerSecond:
            return f"""{self.decimal_fractions_per_second} /s"""
        
        return f'{self.__value}'


    def get_unit_abbreviation(self, unit_abbreviation: RatioChangeRateUnits = RatioChangeRateUnits.DecimalFractionPerSecond) -> string:
        """
        Get RatioChangeRate unit abbreviation.
        Note! the default abbreviation for RatioChangeRate is DecimalFractionPerSecond.
        To specify the unit abbreviation set the 'unit_abbreviation' parameter.
        """
        
        if unit_abbreviation == RatioChangeRateUnits.PercentPerSecond:
            return """%/s"""
        
        if unit_abbreviation == RatioChangeRateUnits.DecimalFractionPerSecond:
            return """/s"""
        

    def __str__(self):
        return self.to_string()


    def __add__(self, other):
        if not isinstance(other, RatioChangeRate):
            raise TypeError("unsupported operand type(s) for +: 'RatioChangeRate' and '{}'".format(type(other).__name__))
        return RatioChangeRate(self.__value + other.__value)


    def __mul__(self, other):
        if not isinstance(other, RatioChangeRate):
            raise TypeError("unsupported operand type(s) for *: 'RatioChangeRate' and '{}'".format(type(other).__name__))
        return RatioChangeRate(self.__value * other.__value)


    def __sub__(self, other):
        if not isinstance(other, RatioChangeRate):
            raise TypeError("unsupported operand type(s) for -: 'RatioChangeRate' and '{}'".format(type(other).__name__))
        return RatioChangeRate(self.__value - other.__value)


    def __truediv__(self, other):
        if not isinstance(other, RatioChangeRate):
            raise TypeError("unsupported operand type(s) for /: 'RatioChangeRate' and '{}'".format(type(other).__name__))
        return RatioChangeRate(self.__value / other.__value)


    def __mod__(self, other):
        if not isinstance(other, RatioChangeRate):
            raise TypeError("unsupported operand type(s) for %: 'RatioChangeRate' and '{}'".format(type(other).__name__))
        return RatioChangeRate(self.__value % other.__value)


    def __pow__(self, other):
        if not isinstance(other, RatioChangeRate):
            raise TypeError("unsupported operand type(s) for **: 'RatioChangeRate' and '{}'".format(type(other).__name__))
        return RatioChangeRate(self.__value ** other.__value)


    def __eq__(self, other):
        if not isinstance(other, RatioChangeRate):
            raise TypeError("unsupported operand type(s) for ==: 'RatioChangeRate' and '{}'".format(type(other).__name__))
        return self.__value == other.__value


    def __lt__(self, other):
        if not isinstance(other, RatioChangeRate):
            raise TypeError("unsupported operand type(s) for <: 'RatioChangeRate' and '{}'".format(type(other).__name__))
        return self.__value < other.__value


    def __gt__(self, other):
        if not isinstance(other, RatioChangeRate):
            raise TypeError("unsupported operand type(s) for >: 'RatioChangeRate' and '{}'".format(type(other).__name__))
        return self.__value > other.__value


    def __le__(self, other):
        if not isinstance(other, RatioChangeRate):
            raise TypeError("unsupported operand type(s) for <=: 'RatioChangeRate' and '{}'".format(type(other).__name__))
        return self.__value <= other.__value


    def __ge__(self, other):
        if not isinstance(other, RatioChangeRate):
            raise TypeError("unsupported operand type(s) for >=: 'RatioChangeRate' and '{}'".format(type(other).__name__))
        return self.__value >= other.__value


from enum import Enum
import math
import string


class IrradianceUnits(Enum):
        """
            IrradianceUnits enumeration
        """
        
        WattPerSquareMeter = 'watt_per_square_meter'
        """
            
        """
        
        WattPerSquareCentimeter = 'watt_per_square_centimeter'
        """
            
        """
        
        PicowattPerSquareMeter = 'picowatt_per_square_meter'
        """
            
        """
        
        NanowattPerSquareMeter = 'nanowatt_per_square_meter'
        """
            
        """
        
        MicrowattPerSquareMeter = 'microwatt_per_square_meter'
        """
            
        """
        
        MilliwattPerSquareMeter = 'milliwatt_per_square_meter'
        """
            
        """
        
        KilowattPerSquareMeter = 'kilowatt_per_square_meter'
        """
            
        """
        
        MegawattPerSquareMeter = 'megawatt_per_square_meter'
        """
            
        """
        
        PicowattPerSquareCentimeter = 'picowatt_per_square_centimeter'
        """
            
        """
        
        NanowattPerSquareCentimeter = 'nanowatt_per_square_centimeter'
        """
            
        """
        
        MicrowattPerSquareCentimeter = 'microwatt_per_square_centimeter'
        """
            
        """
        
        MilliwattPerSquareCentimeter = 'milliwatt_per_square_centimeter'
        """
            
        """
        
        KilowattPerSquareCentimeter = 'kilowatt_per_square_centimeter'
        """
            
        """
        
        MegawattPerSquareCentimeter = 'megawatt_per_square_centimeter'
        """
            
        """
        

class Irradiance:
    """
    Irradiance is the intensity of ultraviolet (UV) or visible light incident on a surface.

    Args:
        value (float): The value.
        from_unit (IrradianceUnits): The Irradiance unit to create from, The default unit is WattPerSquareMeter
    """
    def __init__(self, value: float, from_unit: IrradianceUnits = IrradianceUnits.WattPerSquareMeter):
        if math.isnan(value):
            raise ValueError('Invalid unit: value is NaN')
        self.__value = self.__convert_to_base(value, from_unit)
        
        self.__watts_per_square_meter = None
        
        self.__watts_per_square_centimeter = None
        
        self.__picowatts_per_square_meter = None
        
        self.__nanowatts_per_square_meter = None
        
        self.__microwatts_per_square_meter = None
        
        self.__milliwatts_per_square_meter = None
        
        self.__kilowatts_per_square_meter = None
        
        self.__megawatts_per_square_meter = None
        
        self.__picowatts_per_square_centimeter = None
        
        self.__nanowatts_per_square_centimeter = None
        
        self.__microwatts_per_square_centimeter = None
        
        self.__milliwatts_per_square_centimeter = None
        
        self.__kilowatts_per_square_centimeter = None
        
        self.__megawatts_per_square_centimeter = None
        

    def __convert_from_base(self, from_unit: IrradianceUnits) -> float:
        value = self.__value
        
        if from_unit == IrradianceUnits.WattPerSquareMeter:
            return (value)
        
        if from_unit == IrradianceUnits.WattPerSquareCentimeter:
            return (value * 0.0001)
        
        if from_unit == IrradianceUnits.PicowattPerSquareMeter:
            return ((value) / 1e-12)
        
        if from_unit == IrradianceUnits.NanowattPerSquareMeter:
            return ((value) / 1e-09)
        
        if from_unit == IrradianceUnits.MicrowattPerSquareMeter:
            return ((value) / 1e-06)
        
        if from_unit == IrradianceUnits.MilliwattPerSquareMeter:
            return ((value) / 0.001)
        
        if from_unit == IrradianceUnits.KilowattPerSquareMeter:
            return ((value) / 1000.0)
        
        if from_unit == IrradianceUnits.MegawattPerSquareMeter:
            return ((value) / 1000000.0)
        
        if from_unit == IrradianceUnits.PicowattPerSquareCentimeter:
            return ((value * 0.0001) / 1e-12)
        
        if from_unit == IrradianceUnits.NanowattPerSquareCentimeter:
            return ((value * 0.0001) / 1e-09)
        
        if from_unit == IrradianceUnits.MicrowattPerSquareCentimeter:
            return ((value * 0.0001) / 1e-06)
        
        if from_unit == IrradianceUnits.MilliwattPerSquareCentimeter:
            return ((value * 0.0001) / 0.001)
        
        if from_unit == IrradianceUnits.KilowattPerSquareCentimeter:
            return ((value * 0.0001) / 1000.0)
        
        if from_unit == IrradianceUnits.MegawattPerSquareCentimeter:
            return ((value * 0.0001) / 1000000.0)
        
        return None


    def __convert_to_base(self, value: float, to_unit: IrradianceUnits) -> float:
        
        if to_unit == IrradianceUnits.WattPerSquareMeter:
            return (value)
        
        if to_unit == IrradianceUnits.WattPerSquareCentimeter:
            return (value * 10000)
        
        if to_unit == IrradianceUnits.PicowattPerSquareMeter:
            return ((value) * 1e-12)
        
        if to_unit == IrradianceUnits.NanowattPerSquareMeter:
            return ((value) * 1e-09)
        
        if to_unit == IrradianceUnits.MicrowattPerSquareMeter:
            return ((value) * 1e-06)
        
        if to_unit == IrradianceUnits.MilliwattPerSquareMeter:
            return ((value) * 0.001)
        
        if to_unit == IrradianceUnits.KilowattPerSquareMeter:
            return ((value) * 1000.0)
        
        if to_unit == IrradianceUnits.MegawattPerSquareMeter:
            return ((value) * 1000000.0)
        
        if to_unit == IrradianceUnits.PicowattPerSquareCentimeter:
            return ((value * 10000) * 1e-12)
        
        if to_unit == IrradianceUnits.NanowattPerSquareCentimeter:
            return ((value * 10000) * 1e-09)
        
        if to_unit == IrradianceUnits.MicrowattPerSquareCentimeter:
            return ((value * 10000) * 1e-06)
        
        if to_unit == IrradianceUnits.MilliwattPerSquareCentimeter:
            return ((value * 10000) * 0.001)
        
        if to_unit == IrradianceUnits.KilowattPerSquareCentimeter:
            return ((value * 10000) * 1000.0)
        
        if to_unit == IrradianceUnits.MegawattPerSquareCentimeter:
            return ((value * 10000) * 1000000.0)
        
        return None


    @property
    def base_value(self) -> float:
        return self.__value

    
    @staticmethod
    def from_watts_per_square_meter(watts_per_square_meter: float):
        """
        Create a new instance of Irradiance from a value in watts_per_square_meter.

        

        :param meters: The Irradiance value in watts_per_square_meter.
        :type watts_per_square_meter: float
        :return: A new instance of Irradiance.
        :rtype: Irradiance
        """
        return Irradiance(watts_per_square_meter, IrradianceUnits.WattPerSquareMeter)

    
    @staticmethod
    def from_watts_per_square_centimeter(watts_per_square_centimeter: float):
        """
        Create a new instance of Irradiance from a value in watts_per_square_centimeter.

        

        :param meters: The Irradiance value in watts_per_square_centimeter.
        :type watts_per_square_centimeter: float
        :return: A new instance of Irradiance.
        :rtype: Irradiance
        """
        return Irradiance(watts_per_square_centimeter, IrradianceUnits.WattPerSquareCentimeter)

    
    @staticmethod
    def from_picowatts_per_square_meter(picowatts_per_square_meter: float):
        """
        Create a new instance of Irradiance from a value in picowatts_per_square_meter.

        

        :param meters: The Irradiance value in picowatts_per_square_meter.
        :type picowatts_per_square_meter: float
        :return: A new instance of Irradiance.
        :rtype: Irradiance
        """
        return Irradiance(picowatts_per_square_meter, IrradianceUnits.PicowattPerSquareMeter)

    
    @staticmethod
    def from_nanowatts_per_square_meter(nanowatts_per_square_meter: float):
        """
        Create a new instance of Irradiance from a value in nanowatts_per_square_meter.

        

        :param meters: The Irradiance value in nanowatts_per_square_meter.
        :type nanowatts_per_square_meter: float
        :return: A new instance of Irradiance.
        :rtype: Irradiance
        """
        return Irradiance(nanowatts_per_square_meter, IrradianceUnits.NanowattPerSquareMeter)

    
    @staticmethod
    def from_microwatts_per_square_meter(microwatts_per_square_meter: float):
        """
        Create a new instance of Irradiance from a value in microwatts_per_square_meter.

        

        :param meters: The Irradiance value in microwatts_per_square_meter.
        :type microwatts_per_square_meter: float
        :return: A new instance of Irradiance.
        :rtype: Irradiance
        """
        return Irradiance(microwatts_per_square_meter, IrradianceUnits.MicrowattPerSquareMeter)

    
    @staticmethod
    def from_milliwatts_per_square_meter(milliwatts_per_square_meter: float):
        """
        Create a new instance of Irradiance from a value in milliwatts_per_square_meter.

        

        :param meters: The Irradiance value in milliwatts_per_square_meter.
        :type milliwatts_per_square_meter: float
        :return: A new instance of Irradiance.
        :rtype: Irradiance
        """
        return Irradiance(milliwatts_per_square_meter, IrradianceUnits.MilliwattPerSquareMeter)

    
    @staticmethod
    def from_kilowatts_per_square_meter(kilowatts_per_square_meter: float):
        """
        Create a new instance of Irradiance from a value in kilowatts_per_square_meter.

        

        :param meters: The Irradiance value in kilowatts_per_square_meter.
        :type kilowatts_per_square_meter: float
        :return: A new instance of Irradiance.
        :rtype: Irradiance
        """
        return Irradiance(kilowatts_per_square_meter, IrradianceUnits.KilowattPerSquareMeter)

    
    @staticmethod
    def from_megawatts_per_square_meter(megawatts_per_square_meter: float):
        """
        Create a new instance of Irradiance from a value in megawatts_per_square_meter.

        

        :param meters: The Irradiance value in megawatts_per_square_meter.
        :type megawatts_per_square_meter: float
        :return: A new instance of Irradiance.
        :rtype: Irradiance
        """
        return Irradiance(megawatts_per_square_meter, IrradianceUnits.MegawattPerSquareMeter)

    
    @staticmethod
    def from_picowatts_per_square_centimeter(picowatts_per_square_centimeter: float):
        """
        Create a new instance of Irradiance from a value in picowatts_per_square_centimeter.

        

        :param meters: The Irradiance value in picowatts_per_square_centimeter.
        :type picowatts_per_square_centimeter: float
        :return: A new instance of Irradiance.
        :rtype: Irradiance
        """
        return Irradiance(picowatts_per_square_centimeter, IrradianceUnits.PicowattPerSquareCentimeter)

    
    @staticmethod
    def from_nanowatts_per_square_centimeter(nanowatts_per_square_centimeter: float):
        """
        Create a new instance of Irradiance from a value in nanowatts_per_square_centimeter.

        

        :param meters: The Irradiance value in nanowatts_per_square_centimeter.
        :type nanowatts_per_square_centimeter: float
        :return: A new instance of Irradiance.
        :rtype: Irradiance
        """
        return Irradiance(nanowatts_per_square_centimeter, IrradianceUnits.NanowattPerSquareCentimeter)

    
    @staticmethod
    def from_microwatts_per_square_centimeter(microwatts_per_square_centimeter: float):
        """
        Create a new instance of Irradiance from a value in microwatts_per_square_centimeter.

        

        :param meters: The Irradiance value in microwatts_per_square_centimeter.
        :type microwatts_per_square_centimeter: float
        :return: A new instance of Irradiance.
        :rtype: Irradiance
        """
        return Irradiance(microwatts_per_square_centimeter, IrradianceUnits.MicrowattPerSquareCentimeter)

    
    @staticmethod
    def from_milliwatts_per_square_centimeter(milliwatts_per_square_centimeter: float):
        """
        Create a new instance of Irradiance from a value in milliwatts_per_square_centimeter.

        

        :param meters: The Irradiance value in milliwatts_per_square_centimeter.
        :type milliwatts_per_square_centimeter: float
        :return: A new instance of Irradiance.
        :rtype: Irradiance
        """
        return Irradiance(milliwatts_per_square_centimeter, IrradianceUnits.MilliwattPerSquareCentimeter)

    
    @staticmethod
    def from_kilowatts_per_square_centimeter(kilowatts_per_square_centimeter: float):
        """
        Create a new instance of Irradiance from a value in kilowatts_per_square_centimeter.

        

        :param meters: The Irradiance value in kilowatts_per_square_centimeter.
        :type kilowatts_per_square_centimeter: float
        :return: A new instance of Irradiance.
        :rtype: Irradiance
        """
        return Irradiance(kilowatts_per_square_centimeter, IrradianceUnits.KilowattPerSquareCentimeter)

    
    @staticmethod
    def from_megawatts_per_square_centimeter(megawatts_per_square_centimeter: float):
        """
        Create a new instance of Irradiance from a value in megawatts_per_square_centimeter.

        

        :param meters: The Irradiance value in megawatts_per_square_centimeter.
        :type megawatts_per_square_centimeter: float
        :return: A new instance of Irradiance.
        :rtype: Irradiance
        """
        return Irradiance(megawatts_per_square_centimeter, IrradianceUnits.MegawattPerSquareCentimeter)

    
    @property
    def watts_per_square_meter(self) -> float:
        """
        
        """
        if self.__watts_per_square_meter != None:
            return self.__watts_per_square_meter
        self.__watts_per_square_meter = self.__convert_from_base(IrradianceUnits.WattPerSquareMeter)
        return self.__watts_per_square_meter

    
    @property
    def watts_per_square_centimeter(self) -> float:
        """
        
        """
        if self.__watts_per_square_centimeter != None:
            return self.__watts_per_square_centimeter
        self.__watts_per_square_centimeter = self.__convert_from_base(IrradianceUnits.WattPerSquareCentimeter)
        return self.__watts_per_square_centimeter

    
    @property
    def picowatts_per_square_meter(self) -> float:
        """
        
        """
        if self.__picowatts_per_square_meter != None:
            return self.__picowatts_per_square_meter
        self.__picowatts_per_square_meter = self.__convert_from_base(IrradianceUnits.PicowattPerSquareMeter)
        return self.__picowatts_per_square_meter

    
    @property
    def nanowatts_per_square_meter(self) -> float:
        """
        
        """
        if self.__nanowatts_per_square_meter != None:
            return self.__nanowatts_per_square_meter
        self.__nanowatts_per_square_meter = self.__convert_from_base(IrradianceUnits.NanowattPerSquareMeter)
        return self.__nanowatts_per_square_meter

    
    @property
    def microwatts_per_square_meter(self) -> float:
        """
        
        """
        if self.__microwatts_per_square_meter != None:
            return self.__microwatts_per_square_meter
        self.__microwatts_per_square_meter = self.__convert_from_base(IrradianceUnits.MicrowattPerSquareMeter)
        return self.__microwatts_per_square_meter

    
    @property
    def milliwatts_per_square_meter(self) -> float:
        """
        
        """
        if self.__milliwatts_per_square_meter != None:
            return self.__milliwatts_per_square_meter
        self.__milliwatts_per_square_meter = self.__convert_from_base(IrradianceUnits.MilliwattPerSquareMeter)
        return self.__milliwatts_per_square_meter

    
    @property
    def kilowatts_per_square_meter(self) -> float:
        """
        
        """
        if self.__kilowatts_per_square_meter != None:
            return self.__kilowatts_per_square_meter
        self.__kilowatts_per_square_meter = self.__convert_from_base(IrradianceUnits.KilowattPerSquareMeter)
        return self.__kilowatts_per_square_meter

    
    @property
    def megawatts_per_square_meter(self) -> float:
        """
        
        """
        if self.__megawatts_per_square_meter != None:
            return self.__megawatts_per_square_meter
        self.__megawatts_per_square_meter = self.__convert_from_base(IrradianceUnits.MegawattPerSquareMeter)
        return self.__megawatts_per_square_meter

    
    @property
    def picowatts_per_square_centimeter(self) -> float:
        """
        
        """
        if self.__picowatts_per_square_centimeter != None:
            return self.__picowatts_per_square_centimeter
        self.__picowatts_per_square_centimeter = self.__convert_from_base(IrradianceUnits.PicowattPerSquareCentimeter)
        return self.__picowatts_per_square_centimeter

    
    @property
    def nanowatts_per_square_centimeter(self) -> float:
        """
        
        """
        if self.__nanowatts_per_square_centimeter != None:
            return self.__nanowatts_per_square_centimeter
        self.__nanowatts_per_square_centimeter = self.__convert_from_base(IrradianceUnits.NanowattPerSquareCentimeter)
        return self.__nanowatts_per_square_centimeter

    
    @property
    def microwatts_per_square_centimeter(self) -> float:
        """
        
        """
        if self.__microwatts_per_square_centimeter != None:
            return self.__microwatts_per_square_centimeter
        self.__microwatts_per_square_centimeter = self.__convert_from_base(IrradianceUnits.MicrowattPerSquareCentimeter)
        return self.__microwatts_per_square_centimeter

    
    @property
    def milliwatts_per_square_centimeter(self) -> float:
        """
        
        """
        if self.__milliwatts_per_square_centimeter != None:
            return self.__milliwatts_per_square_centimeter
        self.__milliwatts_per_square_centimeter = self.__convert_from_base(IrradianceUnits.MilliwattPerSquareCentimeter)
        return self.__milliwatts_per_square_centimeter

    
    @property
    def kilowatts_per_square_centimeter(self) -> float:
        """
        
        """
        if self.__kilowatts_per_square_centimeter != None:
            return self.__kilowatts_per_square_centimeter
        self.__kilowatts_per_square_centimeter = self.__convert_from_base(IrradianceUnits.KilowattPerSquareCentimeter)
        return self.__kilowatts_per_square_centimeter

    
    @property
    def megawatts_per_square_centimeter(self) -> float:
        """
        
        """
        if self.__megawatts_per_square_centimeter != None:
            return self.__megawatts_per_square_centimeter
        self.__megawatts_per_square_centimeter = self.__convert_from_base(IrradianceUnits.MegawattPerSquareCentimeter)
        return self.__megawatts_per_square_centimeter

    
    def to_string(self, unit: IrradianceUnits = IrradianceUnits.WattPerSquareMeter) -> string:
        """
        Format the Irradiance to string.
        Note! the default format for Irradiance is WattPerSquareMeter.
        To specify the unit format set the 'unit' parameter.
        """
        
        if unit == IrradianceUnits.WattPerSquareMeter:
            return f"""{self.watts_per_square_meter} W/m²"""
        
        if unit == IrradianceUnits.WattPerSquareCentimeter:
            return f"""{self.watts_per_square_centimeter} W/cm²"""
        
        if unit == IrradianceUnits.PicowattPerSquareMeter:
            return f"""{self.picowatts_per_square_meter} """
        
        if unit == IrradianceUnits.NanowattPerSquareMeter:
            return f"""{self.nanowatts_per_square_meter} """
        
        if unit == IrradianceUnits.MicrowattPerSquareMeter:
            return f"""{self.microwatts_per_square_meter} """
        
        if unit == IrradianceUnits.MilliwattPerSquareMeter:
            return f"""{self.milliwatts_per_square_meter} """
        
        if unit == IrradianceUnits.KilowattPerSquareMeter:
            return f"""{self.kilowatts_per_square_meter} """
        
        if unit == IrradianceUnits.MegawattPerSquareMeter:
            return f"""{self.megawatts_per_square_meter} """
        
        if unit == IrradianceUnits.PicowattPerSquareCentimeter:
            return f"""{self.picowatts_per_square_centimeter} """
        
        if unit == IrradianceUnits.NanowattPerSquareCentimeter:
            return f"""{self.nanowatts_per_square_centimeter} """
        
        if unit == IrradianceUnits.MicrowattPerSquareCentimeter:
            return f"""{self.microwatts_per_square_centimeter} """
        
        if unit == IrradianceUnits.MilliwattPerSquareCentimeter:
            return f"""{self.milliwatts_per_square_centimeter} """
        
        if unit == IrradianceUnits.KilowattPerSquareCentimeter:
            return f"""{self.kilowatts_per_square_centimeter} """
        
        if unit == IrradianceUnits.MegawattPerSquareCentimeter:
            return f"""{self.megawatts_per_square_centimeter} """
        
        return f'{self.__value}'


    def get_unit_abbreviation(self, unit_abbreviation: IrradianceUnits = IrradianceUnits.WattPerSquareMeter) -> string:
        """
        Get Irradiance unit abbreviation.
        Note! the default abbreviation for Irradiance is WattPerSquareMeter.
        To specify the unit abbreviation set the 'unit_abbreviation' parameter.
        """
        
        if unit_abbreviation == IrradianceUnits.WattPerSquareMeter:
            return """W/m²"""
        
        if unit_abbreviation == IrradianceUnits.WattPerSquareCentimeter:
            return """W/cm²"""
        
        if unit_abbreviation == IrradianceUnits.PicowattPerSquareMeter:
            return """"""
        
        if unit_abbreviation == IrradianceUnits.NanowattPerSquareMeter:
            return """"""
        
        if unit_abbreviation == IrradianceUnits.MicrowattPerSquareMeter:
            return """"""
        
        if unit_abbreviation == IrradianceUnits.MilliwattPerSquareMeter:
            return """"""
        
        if unit_abbreviation == IrradianceUnits.KilowattPerSquareMeter:
            return """"""
        
        if unit_abbreviation == IrradianceUnits.MegawattPerSquareMeter:
            return """"""
        
        if unit_abbreviation == IrradianceUnits.PicowattPerSquareCentimeter:
            return """"""
        
        if unit_abbreviation == IrradianceUnits.NanowattPerSquareCentimeter:
            return """"""
        
        if unit_abbreviation == IrradianceUnits.MicrowattPerSquareCentimeter:
            return """"""
        
        if unit_abbreviation == IrradianceUnits.MilliwattPerSquareCentimeter:
            return """"""
        
        if unit_abbreviation == IrradianceUnits.KilowattPerSquareCentimeter:
            return """"""
        
        if unit_abbreviation == IrradianceUnits.MegawattPerSquareCentimeter:
            return """"""
        

    def __str__(self):
        return self.to_string()


    def __add__(self, other):
        if not isinstance(other, Irradiance):
            raise TypeError("unsupported operand type(s) for +: 'Irradiance' and '{}'".format(type(other).__name__))
        return Irradiance(self.__value + other.__value)


    def __mul__(self, other):
        if not isinstance(other, Irradiance):
            raise TypeError("unsupported operand type(s) for *: 'Irradiance' and '{}'".format(type(other).__name__))
        return Irradiance(self.__value * other.__value)


    def __sub__(self, other):
        if not isinstance(other, Irradiance):
            raise TypeError("unsupported operand type(s) for -: 'Irradiance' and '{}'".format(type(other).__name__))
        return Irradiance(self.__value - other.__value)


    def __truediv__(self, other):
        if not isinstance(other, Irradiance):
            raise TypeError("unsupported operand type(s) for /: 'Irradiance' and '{}'".format(type(other).__name__))
        return Irradiance(self.__value / other.__value)


    def __mod__(self, other):
        if not isinstance(other, Irradiance):
            raise TypeError("unsupported operand type(s) for %: 'Irradiance' and '{}'".format(type(other).__name__))
        return Irradiance(self.__value % other.__value)


    def __pow__(self, other):
        if not isinstance(other, Irradiance):
            raise TypeError("unsupported operand type(s) for **: 'Irradiance' and '{}'".format(type(other).__name__))
        return Irradiance(self.__value ** other.__value)


    def __eq__(self, other):
        if not isinstance(other, Irradiance):
            raise TypeError("unsupported operand type(s) for ==: 'Irradiance' and '{}'".format(type(other).__name__))
        return self.__value == other.__value


    def __lt__(self, other):
        if not isinstance(other, Irradiance):
            raise TypeError("unsupported operand type(s) for <: 'Irradiance' and '{}'".format(type(other).__name__))
        return self.__value < other.__value


    def __gt__(self, other):
        if not isinstance(other, Irradiance):
            raise TypeError("unsupported operand type(s) for >: 'Irradiance' and '{}'".format(type(other).__name__))
        return self.__value > other.__value


    def __le__(self, other):
        if not isinstance(other, Irradiance):
            raise TypeError("unsupported operand type(s) for <=: 'Irradiance' and '{}'".format(type(other).__name__))
        return self.__value <= other.__value


    def __ge__(self, other):
        if not isinstance(other, Irradiance):
            raise TypeError("unsupported operand type(s) for >=: 'Irradiance' and '{}'".format(type(other).__name__))
        return self.__value >= other.__value